@@ -57,7 +57,7 @@ enum PINLINE;
 #define STCout          0x1000LL        // out parameter
 #define STClazy         0x2000LL        // lazy parameter
 #define STCforeach      0x4000LL        // variable for foreach loop
-#define STCvariadic     0x10000LL       // variadic function argument
+#define STCvariadic     0x10000LL       // the 'variadic' parameter in: T foo(T a, U b, V variadic...)
 #define STCctorinit     0x20000LL       // can only be set inside constructor
 #define STCtemplateparameter  0x40000LL // template parameter
 #define STCscope        0x80000LL
@@ -90,9 +90,11 @@ enum PINLINE;
 #define STCrvalue        0x20000000000LL // force rvalue for variables
 #define STCnogc          0x40000000000LL // @nogc
 #define STCvolatile      0x80000000000LL // destined for volatile in the back end
-#define STCreturn        0x100000000000LL // 'return ref' for function parameters
+#define STCreturn        0x100000000000LL // 'return ref' or 'return scope' for function parameters
 #define STCautoref       0x200000000000LL // Mark for the already deduced 'auto ref' parameter
 #define STCinference     0x400000000000LL // do attribute inference
+#define STCexptemp       0x800000000000LL // temporary variable that has lifetime restricted to an expression
+#define STCmaybescope    0x1000000000000LL // parameter might be 'scope'
 
 const StorageClass STCStorageClass = (STCauto | STCscope | STCstatic | STCextern | STCconst | STCfinal |
     STCabstract | STCsynchronized | STCdeprecated | STCoverride | STClazy | STCalias |
@@ -195,7 +197,7 @@ class AliasDeclaration : public Declaration
 public:
     Dsymbol *aliassym;
     Dsymbol *overnext;          // next in overload list
-    Dsymbol *import;            // !=NULL if unresolved internal alias for selective import
+    Dsymbol *_import;           // !=NULL if unresolved internal alias for selective import
 
     Dsymbol *syntaxCopy(Dsymbol *);
     void semantic(Scope *sc);
@@ -205,6 +207,7 @@ public:
     Type *getType();
     Dsymbol *toAlias();
     Dsymbol *toAlias2();
+    bool isOverloadable();
 
     AliasDeclaration *isAliasDeclaration() { return this; }
     void accept(Visitor *v) { v->visit(this); }
@@ -226,6 +229,7 @@ public:
 
     Dsymbol *toAlias();
     Dsymbol *isUnique();
+    bool isOverloadable();
 
     OverDeclaration *isOverDeclaration() { return this; }
     void accept(Visitor *v) { v->visit(this); }
@@ -238,6 +242,7 @@ class VarDeclaration : public Declaration
 public:
     Initializer *_init;
     unsigned offset;
+    unsigned sequenceNumber;     // order the variables are declared
     FuncDeclarations nestedrefs; // referenced by these lexically nested functions
     bool isargptr;              // if parameter that _argptr points to
     structalign_t alignment;
@@ -246,6 +251,8 @@ public:
     bool mynew;                 // it is a class new'd with custom operator new
     int canassign;              // it can be assigned to
     bool overlapped;            // if it is a field and has overlapping
+    bool overlapUnsafe;         // if it is an overlapping field and the overlaps are unsafe
+    bool doNotInferScope;       // do not infer 'scope' for this variable
     unsigned char isdataseg;    // private data for isDataseg
     Dsymbol *aliassym;          // if redone as alias to another symbol
     VarDeclaration *lastVar;    // Linked list of variables for goto-skips-init detection
@@ -277,8 +284,10 @@ public:
     bool hasPointers();
     bool canTakeAddressOf();
     bool needsScopeDtor();
+    bool enclosesLifetimeOf(VarDeclaration *v) const;
     Expression *callScopeDtor(Scope *sc);
     Expression *getConstInitializer(bool needFullType = true);
+    Expression *expandInitializer(Loc loc);
     void checkCtorConstInit();
     bool checkNestedReference(Scope *sc, Loc loc);
     Dsymbol *toAlias();
@@ -481,6 +490,8 @@ void builtin_init();
 #define FUNCFLAGnothrowInprocess   4    // working on determining nothrow
 #define FUNCFLAGnogcInprocess      8    // working on determining @nogc
 #define FUNCFLAGreturnInprocess 0x10    // working on inferring 'return' for parameters
+#define FUNCFLAGinlineScanned   0x20    // function has been scanned for inline possibilities
+#define FUNCFLAGinferScope      0x40    // infer 'scope' for parameters
 
 class FuncDeclaration : public Declaration
 {
@@ -594,15 +605,17 @@ public:
     void semantic3(Scope *sc);
     bool functionSemantic();
     bool functionSemantic3();
+    bool checkForwardRef(Loc loc);
     // called from semantic3
     VarDeclaration *declareThis(Scope *sc, AggregateDeclaration *ad);
     bool equals(RootObject *o);
 
     int overrides(FuncDeclaration *fd);
     int findVtblIndex(Dsymbols *vtbl, int dim);
+    BaseClass *overrideInterface();
     bool overloadInsert(Dsymbol *s);
     FuncDeclaration *overloadExactMatch(Type *t);
-    FuncDeclaration *overloadModMatch(Loc loc, Type *tthis, Type *&t);
+    FuncDeclaration *overloadModMatch(Loc loc, Type *tthis, bool &hasOverloads);
     TemplateDeclaration *findTemplateDeclRoot();
     bool inUnittest();
     MATCH leastAsSpecialized(FuncDeclaration *g);
@@ -659,6 +672,7 @@ public:
 
     static FuncDeclaration *genCfunc(Parameters *args, Type *treturn, const char *name, StorageClass stc=0);
     static FuncDeclaration *genCfunc(Parameters *args, Type *treturn, Identifier *id, StorageClass stc=0);
+    void checkDmain();
 
     FuncDeclaration *isFuncDeclaration() { return this; }
 
@@ -696,6 +710,7 @@ public:
 
     Dsymbol *syntaxCopy(Dsymbol *);
     bool isNested();
+    AggregateDeclaration *isThis();
     bool isVirtual();
     bool addPreInvariant();
     bool addPostInvariant();
