@@ -21,14 +21,20 @@
 
 class Identifier : public RootObject
 {
+private:
+    int value;
+    const char *string;
+    size_t len;
+
 public:
     static Identifier* create(const char *string);
     bool equals(RootObject *o);
     int compare(RootObject *o);
     void print();
     const char *toChars();
+    int getValue() const;
     const char *toHChars2();
-    int dyncast();
+    int dyncast() const;
 
     static StringTable stringtable;
     static Identifier *generateId(const char *prefix);
