@@ -113,6 +113,8 @@ typedef unsigned char TY;       // ENUMTY
 extern int Tsize_t;
 extern int Tptrdiff_t;
 
+#define SIZE_INVALID (~(d_uns64)0)   // error return from size() functions
+
 
 /**
  * type modifiers
@@ -240,13 +242,12 @@ public:
     bool equals(RootObject *o);
     bool equivalent(Type *t);
     // kludge for template.isType()
-    int dyncast() { return DYNCAST_TYPE; }
+    int dyncast() const { return DYNCAST_TYPE; }
     int covariant(Type *t, StorageClass *pstc = NULL);
     const char *toChars();
     char *toPrettyChars(bool QualifyTypes = false);
     static void _init();
 
-    #define SIZE_INVALID (~(d_uns64)0)
     d_uns64 size();
     virtual d_uns64 size(Loc loc);
     virtual unsigned alignsize();
@@ -335,6 +336,7 @@ public:
     virtual bool isZeroInit(Loc loc = Loc());                // if initializer is 0
     Identifier *getTypeInfoIdent();
     virtual void resolve(Loc loc, Scope *sc, Expression **pe, Type **pt, Dsymbol **ps, bool intypeid = false);
+    void resolveExp(Expression *e, Type **pt, Expression **pe, Dsymbol **ps);
     virtual int hasWild() const;
     virtual Expression *toExpression();
     virtual bool hasPointers();
@@ -621,6 +623,7 @@ public:
     void purityLevel();
     bool hasLazyParameters();
     bool parameterEscapes(Parameter *p);
+    StorageClass parameterStorageClass(Parameter *p);
     Type *addStorageClass(StorageClass stc);
 
     /** For each active attribute (ref/const/nogc/etc) call fp with a void* for the
@@ -671,8 +674,7 @@ public:
 
     void resolveTupleIndex(Loc loc, Scope *sc, Dsymbol *s,
         Expression **pe, Type **pt, Dsymbol **ps, RootObject *oindex);
-    void resolveExprType(Loc loc, Scope *sc, Expression *e, size_t i,
-        Expression **pe, Type **pt);
+    Expression *toExpressionHelper(Expression *e, size_t i = 0);
     void resolveHelper(Loc loc, Scope *sc, Dsymbol *s, Dsymbol *scopesym,
         Expression **pe, Type **pt, Dsymbol **ps, bool intypeid = false);
 
@@ -755,12 +757,6 @@ public:
     AliasThisRec att;
     CPPMANGLE cppmangle;
 
-#if IN_LLVM
-    // cache the hasUnalignedFields check
-    // 0 = not checked, 1 = aligned, 2 = unaligned
-    int32_t unaligned;
-#endif
-
     const char *kind();
     d_uns64 size(Loc loc);
     unsigned alignsize();
@@ -777,6 +773,7 @@ public:
     bool needsDestruction() /*const*/;
     bool needsNested();
     bool hasPointers();
+    bool hasVoidInitPointers();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
     unsigned char deduceWild(Type *t, bool isRef);
@@ -816,6 +813,7 @@ public:
     Expression *defaultInit(Loc loc);
     bool isZeroInit(Loc loc);
     bool hasPointers();
+    bool hasVoidInitPointers();
     Type *nextOf();
 
     void accept(Visitor *v) { v->visit(this); }
@@ -826,6 +824,7 @@ class TypeClass : public Type
 public:
     ClassDeclaration *sym;
     AliasThisRec att;
+    CPPMANGLE cppmangle;
 
     const char *kind();
     d_uns64 size(Loc loc) /*const*/;
@@ -907,12 +906,14 @@ public:
     Parameter *syntaxCopy();
     Type *isLazyArray();
     // kludge for template.isType()
-    int dyncast() { return DYNCAST_PARAMETER; }
+    int dyncast() const { return DYNCAST_PARAMETER; }
     virtual void accept(Visitor *v) { v->visit(this); }
 
     static Parameters *arraySyntaxCopy(Parameters *parameters);
     static size_t dim(Parameters *parameters);
     static Parameter *getNth(Parameters *parameters, d_size_t nth, d_size_t *pn = NULL);
+    const char *toChars();
+    bool isCovariant(const Parameter *p) const;
 };
 
 bool arrayTypeCompatible(Loc loc, Type *t1, Type *t2);
