@@ -24,6 +24,18 @@ typedef size_t hash_t;
 
 struct OutBuffer;
 
+enum DYNCAST
+{
+    DYNCAST_OBJECT,
+    DYNCAST_EXPRESSION,
+    DYNCAST_DSYMBOL,
+    DYNCAST_TYPE,
+    DYNCAST_IDENTIFIER,
+    DYNCAST_TUPLE,
+    DYNCAST_PARAMETER,
+    DYNCAST_STATEMENT,
+};
+
 /*
  * Root of our class library.
  */
@@ -52,7 +64,7 @@ public:
      * Used as a replacement for dynamic_cast. Returns a unique number
      * defined by the library user. For Object, the return value is 0.
      */
-    virtual int dyncast();
+    virtual int dyncast() const;
 };
 
 #endif
