@@ -588,6 +588,10 @@ public:
     Identifier *ident;
     VarDeclaration *var;
     Statement *handler;
+
+    // set if semantic processing errors
+    bool errors;
+
     // was generated by the compiler,
     // wasn't present in source code
     bool internalCatch;
