@@ -13,11 +13,16 @@
 #include "mars.h"
 #include "module.h"
 #include "scope.h"
+#include "driver/cl_options.h"
 #include "driver/linker.h"
 #include "driver/toobj.h"
 #include "gen/logger.h"
 #include "gen/modules.h"
 #include "gen/runtime.h"
+#include "llvm/Support/FileSystem.h"
+#include "llvm/Support/Path.h"
+#include "llvm/Support/ToolOutputFile.h"
+#include "llvm/Support/YAMLTraits.h"
 
 /// The module with the frontend-generated C main() definition.
 extern Module *g_entrypointModule;
@@ -27,7 +32,54 @@ extern Module *g_dMainModule;
 
 namespace {
 
-/// Add the linker options metadata flag.
+std::unique_ptr<llvm::tool_output_file>
+createAndSetDiagnosticsOutputFile(IRState &irs, llvm::LLVMContext &ctx,
+                                  llvm::StringRef filename) {
+  std::unique_ptr<llvm::tool_output_file> diagnosticsOutputFile;
+
+#if LDC_LLVM_VER >= 400
+  // Set LLVM Diagnostics outputfile if requested
+  if (opts::saveOptimizationRecord.getNumOccurrences() > 0) {
+    llvm::SmallString<128> diagnosticsFilename;
+    if (!opts::saveOptimizationRecord.empty()) {
+      diagnosticsFilename = opts::saveOptimizationRecord.getValue();
+    } else {
+      diagnosticsFilename = filename;
+      llvm::sys::path::replace_extension(diagnosticsFilename, "opt.yaml");
+    }
+
+    std::error_code EC;
+    diagnosticsOutputFile = llvm::make_unique<llvm::tool_output_file>(
+        diagnosticsFilename, EC, llvm::sys::fs::F_None);
+    if (EC) {
+      irs.dmodule->error("Could not create file %s: %s",
+                         diagnosticsFilename.c_str(), EC.message().c_str());
+      fatal();
+    }
+
+    ctx.setDiagnosticsOutputFile(
+        llvm::make_unique<llvm::yaml::Output>(diagnosticsOutputFile->os()));
+
+    // If there is instrumentation data available, also output function hotness
+    if (!global.params.genInstrProf && global.params.datafileInstrProf) {
+#if LDC_LLVM_VER >= 500
+      ctx.setDiagnosticsHotnessRequested(true);
+#else
+      ctx.setDiagnosticHotnessRequested(true);
+#endif
+    }
+  }
+#endif
+
+  return diagnosticsOutputFile;
+}
+
+} // anonymous namespace
+
+namespace {
+
+#if LDC_LLVM_VER < 500
+/// Add the Linker Options module flag.
 /// If the flag is already present, merge it with the new data.
 void emitLinkerOptions(IRState &irs, llvm::Module &M, llvm::LLVMContext &ctx) {
   if (!M.getModuleFlag("Linker Options")) {
@@ -37,11 +89,6 @@ void emitLinkerOptions(IRState &irs, llvm::Module &M, llvm::LLVMContext &ctx) {
     // Merge the Linker Options with the pre-existing one
     // (this can happen when passing a .bc file on the commandline)
 
-#if LDC_LLVM_VER < 306
-    // Passing a bitcode file on the commandline is not supported for LLVM 3.5.
-    llvm_unreachable(
-        "Merging of Linker Options is not implemented for LLVM 3.5");
-#else
     auto *moduleFlags = M.getModuleFlagsMetadata();
     for (unsigned i = 0, e = moduleFlags->getNumOperands(); i < e; ++i) {
       auto *flag = moduleFlags->getOperand(i);
@@ -70,9 +117,33 @@ void emitLinkerOptions(IRState &irs, llvm::Module &M, llvm::LLVMContext &ctx) {
 
       break;
     }
-#endif
   }
 }
+#else
+/// Add the "llvm.linker.options" metadata.
+/// If the metadata is already present, merge it with the new data.
+void emitLinkerOptions(IRState &irs, llvm::Module &M, llvm::LLVMContext &ctx) {
+  auto *linkerOptionsMD = M.getOrInsertNamedMetadata("llvm.linker.options");
+
+  // Add the new operands in front of the existing ones, such that linker
+  // options of .bc files passed on the cmdline are put _after_ the compiled .d
+  // file.
+
+  // Temporarily store metadata nodes that are already present
+  llvm::SmallVector<llvm::MDNode *, 5> oldMDNodes;
+  for (auto *MD : linkerOptionsMD->operands())
+    oldMDNodes.push_back(MD);
+
+  // Clear the list and add the new metadata nodes.
+  linkerOptionsMD->clearOperands();
+  for (auto *MD : irs.LinkerMetadataArgs)
+    linkerOptionsMD->addOperand(MD);
+
+  // Re-add metadata nodes that were already present
+  for (auto *MD : oldMDNodes)
+    linkerOptionsMD->addOperand(MD);
+}
+#endif
 
 void emitLLVMUsedArray(IRState &irs) {
   if (irs.usedArray.empty()) {
@@ -170,6 +241,10 @@ void CodeGenerator::finishLLModule(Module *m) {
 }
 
 void CodeGenerator::writeAndFreeLLModule(const char *filename) {
+  // Issue #1829: make sure all replaced global variables are replaced
+  // everywhere.
+  ir_->replaceGlobals();
+
   ir_->DBuilder.Finalize();
 
   emitLLVMUsedArray(*ir_);
@@ -180,15 +255,17 @@ void CodeGenerator::writeAndFreeLLModule(const char *filename) {
       ir_->module.getOrInsertNamedMetadata("llvm.ident");
   std::string Version("ldc version ");
   Version.append(global.ldc_version);
-#if LDC_LLVM_VER >= 306
-  llvm::Metadata *IdentNode[] =
-#else
-  llvm::Value *IdentNode[] =
-#endif
-      {llvm::MDString::get(ir_->context(), Version)};
+  llvm::Metadata *IdentNode[] = {llvm::MDString::get(ir_->context(), Version)};
   IdentMetadata->addOperand(llvm::MDNode::get(ir_->context(), IdentNode));
 
+  std::unique_ptr<llvm::tool_output_file> diagnosticsOutputFile =
+      createAndSetDiagnosticsOutputFile(*ir_, context_, filename);
+
   writeModule(&ir_->module, filename);
+
+  if (diagnosticsOutputFile)
+    diagnosticsOutputFile->keep();
+
   delete ir_;
   ir_ = nullptr;
 }
