@@ -10,37 +10,12 @@
 //
 // Wrapper allowing use of LDC as drop-in replacement for DMD.
 //
-// The reason why full command line parsing is required instead of just
-// rewriting the names of a few switches is an annoying impedance mismatch
-// between the way how DMD handles arguments and the LLVM command line library:
-// DMD allows all switches to be specified multiple times – in case of
-// conflicts, the last one takes precedence. There is no easy way to replicate
-// this behavior with LLVM, save parsing the switches and re-emitting a cleaned
-// up string.
+// Most command-line options are passed through to LDC; some with different
+// names or semantics need to be translated.
 //
 // DMD also reads switches from the DFLAGS enviroment variable, if present. This
 // is contrary to what C compilers do, where CFLAGS is usually handled by the
-// build system. Thus, conflicts like mentioned above occur quite frequently in
-// practice in makefiles written for DMD, as DFLAGS is also a natural name for
-// handling flags there.
-//
-// In order to maintain backwards compatibility with earlier versions of LDMD,
-// unknown switches are passed through verbatim to LDC. Finding a better
-// solution for this is tricky, as some of the LLVM arguments can be
-// intentionally specified multiple times to get a certain effect (e.g. pass,
-// linker options).
-//
-// Just as with the old LDMD script, arguments can be passed through unmodified
-// to LDC by using -Csomearg.
-//
-// If maintaining this wrapper is deemed too messy at some point, an alternative
-// would be to either extend the LLVM command line library to support the DMD
-// semantics (unlikely to happen), or to abandon it altogether (except for
-// passing the LLVM-defined flags to the various passes).
-//
-// Note: This program inherited ugly C-style string handling and memory leaks
-// from DMD, but this should not be a problem due to the short-livedness of
-// the process.
+// build system.
 //
 //===----------------------------------------------------------------------===//
 
@@ -75,9 +50,11 @@
 
 namespace ls = llvm::sys;
 
-// We reuse DMD's response file parsing routine for maximum compatibilty - it
+// We reuse DMD's response file parsing routine for maximum compatibility - it
 // handles quotes in a very peculiar way.
 int response_expand(size_t *pargc, char ***pargv);
+
+// in ddmd/root/man.d
 void browse(const char *url);
 
 /**
@@ -182,6 +159,7 @@ Where:\n\
   -deps=<filename> write module dependencies to filename (only imports)\n\
   -fPIC            generate position independent code\n\
   -dip25           implement http://wiki.dlang.org/DIP25 (experimental)\n\
+  -dip1000         implement http://wiki.dlang.org/DIP1000 (experimental)\n\
   -g               add symbolic debug info\n\
   -gc              add symbolic debug info, optimize for non D debuggers\n\
   -gs              always emit stack frame\n"
@@ -208,7 +186,11 @@ Where:\n\
 #if 0
 "  -map             generate linker .map file\n"
 #endif
-"  -noboundscheck   no array bounds checking (deprecated, use -boundscheck=off)\n\
+"  -mcpu=<id>       generate instructions for architecture identified by 'id'\n\
+  -mcpu=?          list all architecture options\n\
+  -mscrtlib=<name> MS C runtime library to reference from main/WinMain/DllMain\n\
+  -mv=<package.module>=<filespec>  use <filespec> as source file for <package.module>\n\
+  -noboundscheck   no array bounds checking (deprecated, use -boundscheck=off)\n\
   -O               optimize\n\
   -o-              do not write object file\n\
   -od=<directory>  write object & library files to directory\n\
@@ -414,7 +396,10 @@ void translateArgs(size_t originalArgc, char **originalArgv,
        */
       else if (strcmp(p + 1, "m32mscoff") == 0) {
         ldcArgs.push_back("-m32");
-      } else if (strcmp(p + 1, "profile") == 0) {
+      }
+      /* -mscrtlib
+       */
+      else if (strcmp(p + 1, "profile") == 0) {
         goto Lnot_in_ldc;
       }
       /* -v
@@ -433,6 +418,20 @@ void translateArgs(size_t originalArgc, char **originalArgv,
         } else {
           goto Lerror;
         }
+      } else if (strcmp(p + 1, "mcpu=?") == 0) {
+        const char *mcpuargs[] = {ldcPath.c_str(), "-mcpu=help", nullptr};
+        execute(ldcPath, mcpuargs);
+        exit(EXIT_SUCCESS);
+      } else if (memcmp(p + 1, "mcpu=", 5) == 0) {
+        if (strcmp(p + 6, "baseline") == 0) {
+          // ignore
+        } else if (strcmp(p + 6, "avx") == 0) {
+          ldcArgs.push_back("-mattr=+avx");
+        } else if (strcmp(p + 6, "native") == 0) {
+          ldcArgs.push_back(p);
+        } else {
+          goto Lerror;
+        }
       } else if (strcmp(p + 1, "transition=?") == 0) {
         const char *transitionargs[] = {ldcPath.c_str(), p, nullptr};
         execute(ldcPath, transitionargs);
@@ -466,6 +465,7 @@ void translateArgs(size_t originalArgc, char **originalArgv,
         ldcArgs.push_back("-Hkeep-all-bodies");
       }
       /* -dip25
+       * -dip1000
        */
       else if (strcmp(p + 1, "lib") == 0) {
         ldcArgs.push_back(p);
@@ -633,12 +633,8 @@ std::string locateBinary(std::string exeName) {
     return path;
   }
 
-#if LDC_LLVM_VER >= 306
   llvm::ErrorOr<std::string> res = ls::findProgramByName(exeName);
   path = res ? res.get() : std::string();
-#else
-  path = ls::FindProgramByName(exeName);
-#endif
   if (ls::fs::can_execute(path)) {
     return path;
   }
