@@ -21,14 +21,17 @@
 #include "ddmd/target.h"
 #include "driver/cache.h"
 #include "driver/cl_options.h"
+#include "driver/cl_options_sanitizers.h"
 #include "driver/codegenerator.h"
 #include "driver/configfile.h"
+#include "driver/dcomputecodegenerator.h"
 #include "driver/exe_path.h"
 #include "driver/ldc-version.h"
 #include "driver/linker.h"
 #include "driver/targetmachine.h"
 #include "gen/cl_helpers.h"
 #include "gen/irstate.h"
+#include "gen/ldctraits.h"
 #include "gen/linkage.h"
 #include "gen/llvm.h"
 #include "gen/llvmhelpers.h"
@@ -39,6 +42,7 @@
 #include "gen/optimizer.h"
 #include "gen/passes/Passes.h"
 #include "gen/runtime.h"
+#include "gen/uda.h"
 #include "gen/abi.h"
 #include "llvm/InitializePasses.h"
 #include "llvm/LinkAllPasses.h"
@@ -52,9 +56,7 @@
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Target/TargetMachine.h"
-#if LDC_LLVM_VER >= 306
 #include "llvm/Target/TargetSubtargetInfo.h"
-#endif
 #include "llvm/LinkAllIR.h"
 #include "llvm/IR/LLVMContext.h"
 #include <assert.h>
@@ -80,10 +82,9 @@ using namespace opts;
 
 extern void getenv_setargv(const char *envvar, int *pargc, char ***pargv);
 
-static cl::opt<bool>
-    noDefaultLib("nodefaultlib",
-                 cl::desc("Don't add a default library for linking implicitly"),
-                 cl::ZeroOrMore, cl::Hidden);
+static cl::opt<bool> noDefaultLib(
+    "nodefaultlib", cl::ZeroOrMore, cl::Hidden,
+    cl::desc("Don't add a default library for linking implicitly"));
 
 static StringsAdapter impPathsStore("I", global.params.imppath);
 static cl::list<std::string, StringsAdapter>
@@ -92,19 +93,16 @@ static cl::list<std::string, StringsAdapter>
                 cl::Prefix);
 
 static cl::opt<std::string>
-    defaultLib("defaultlib",
-               cl::desc("Default libraries to link with (overrides previous)"),
-               cl::value_desc("lib1,lib2,..."), cl::ZeroOrMore);
+    defaultLib("defaultlib", cl::ZeroOrMore, cl::value_desc("lib1,lib2,..."),
+               cl::desc("Default libraries to link with (overrides previous)"));
 
 static cl::opt<std::string> debugLib(
-    "debuglib",
-    cl::desc("Debug versions of default libraries (overrides previous)"),
-    cl::value_desc("lib1,lib2,..."), cl::ZeroOrMore);
+    "debuglib", cl::ZeroOrMore, cl::value_desc("lib1,lib2,..."),
+    cl::desc("Debug versions of default libraries (overrides previous)"));
 
 static cl::opt<bool> linkDebugLib(
-    "link-debuglib",
-    cl::desc("Link with libraries specified in -debuglib, not -defaultlib"),
-    cl::ZeroOrMore);
+    "link-debuglib", cl::ZeroOrMore,
+    cl::desc("Link with libraries specified in -debuglib, not -defaultlib"));
 
 #if LDC_LLVM_VER >= 309
 static inline llvm::Optional<llvm::Reloc::Model> getRelocModel() {
@@ -118,33 +116,45 @@ static inline llvm::Optional<llvm::Reloc::Model> getRelocModel() {
 static inline llvm::Reloc::Model getRelocModel() { return mRelocModel; }
 #endif
 
-void printVersion() {
-  printf("LDC - the LLVM D compiler (%s):\n", global.ldc_version);
-  printf("  based on DMD %s and LLVM %s\n", global.version,
-         global.llvm_version);
-  printf("  built with %s\n", ldc::built_with_Dcompiler_version);
+// This function exits the program.
+void printVersion(llvm::raw_ostream &OS) {
+  OS << "LDC - the LLVM D compiler (" << global.ldc_version << "):\n";
+  OS << "  based on DMD " << global.version << " and LLVM " << global.llvm_version << "\n";
+  OS << "  built with " << ldc::built_with_Dcompiler_version << "\n";
 #if defined(__has_feature)
 #if __has_feature(address_sanitizer)
-  printf("  compiled with address sanitizer enabled\n");
+  OS << "  compiled with address sanitizer enabled\n";
 #endif
 #endif
-  printf("  Default target: %s\n", llvm::sys::getDefaultTargetTriple().c_str());
+  OS << "  Default target: " << llvm::sys::getDefaultTargetTriple() << "\n";
   std::string CPU = llvm::sys::getHostCPUName();
   if (CPU == "generic") {
     CPU = "(unknown)";
   }
-  printf("  Host CPU: %s\n", CPU.c_str());
-  printf("  http://dlang.org - http://wiki.dlang.org/LDC\n");
-  printf("\n");
+  OS << "  Host CPU: " << CPU << "\n";
+  OS << "  http://dlang.org - http://wiki.dlang.org/LDC\n";
+  OS << "\n";
 
   // Without explicitly flushing here, only the target list is visible when
   // redirecting stdout to a file.
-  fflush(stdout);
+  OS.flush();
+
+  llvm::TargetRegistry::printRegisteredTargetsForVersion(
+#if LDC_LLVM_VER >= 600
+    OS
+#endif
+    );
 
-  llvm::TargetRegistry::printRegisteredTargetsForVersion();
   exit(EXIT_SUCCESS);
 }
 
+// This function exits the program.
+void printVersionStdout() {
+  printVersion(llvm::outs());
+  assert(false);
+}
+
+
 namespace {
 
 // Helper function to handle -d-debug=* and -d-version=*
@@ -187,8 +197,6 @@ void processTransitions(std::vector<std::string> &list) {
              "  =field,3449    list all non-mutable fields which occupy an "
              "object instance\n"
              "  =import,10378  revert to single phase name lookup\n"
-             "  =safe          shows places with hidden change in semantics "
-             "needed for better @safe guarantees\n"
              "  =tls           list all variables going into thread local "
              "storage\n");
       exit(EXIT_SUCCESS);
@@ -206,8 +214,6 @@ void processTransitions(std::vector<std::string> &list) {
       global.params.vfield = true;
     } else if (i == "import" || i == "10378") {
       global.params.bug10378 = true;
-    } else if (i == "safe") {
-      global.params.vsafe = true;
     } else if (i == "tls") {
       global.params.vtls = true;
     } else {
@@ -229,7 +235,13 @@ void initFromPathString(const char *&dest, const cl::opt<std::string> &src) {
   dest = nullptr;
   if (src.getNumOccurrences() != 0) {
     if (src.empty()) {
-      error(Loc(), "Expected argument to '-%s'", src.ArgStr);
+      error(Loc(), "Expected argument to '-%s'",
+#if LDC_LLVM_VER >= 308
+            src.ArgStr.str().c_str()
+#else
+            src.ArgStr
+#endif
+      );
     }
     dest = dupPathString(src);
   }
@@ -342,14 +354,16 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   // just ignore errors for now, they are still printed
   cfg_file.read(explicitConfFile, cfg_triple.c_str());
 
-  // insert switches from config file before all explicit ones
-  allArguments.insert(allArguments.begin() + 1, cfg_file.switches_begin(),
-                      cfg_file.switches_end());
+  cfg_file.extendCommandLine(allArguments);
 
   // finalize by expanding response files specified in config file
   expandResponseFiles(allocator, allArguments);
 
+#if LDC_LLVM_VER >= 600
   cl::SetVersionPrinter(&printVersion);
+#else
+  cl::SetVersionPrinter(&printVersionStdout);
+#endif
 
   opts::hideLLVMOptions();
   opts::createClashingOptions();
@@ -360,6 +374,19 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
 
   helpOnly = mCPU == "help" ||
              (std::find(mAttrs.begin(), mAttrs.end(), "help") != mAttrs.end());
+  if (helpOnly) {
+    auto triple = llvm::Triple(cfg_triple);
+    std::string errMsg;
+    if (auto target = lookupTarget("", triple, errMsg)) {
+      llvm::errs() << "Targeting " << target->getName() << ". ";
+      // this prints the available CPUs and features of the target to stderr...
+      target->createMCSubtargetInfo(cfg_triple, "help", "");
+    } else {
+      error(Loc(), "%s", errMsg.c_str());
+      fatal();
+    }
+    return;
+  }
 
   // Print some information if -v was passed
   // - path to compiler binary
@@ -369,7 +396,7 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
     fprintf(global.stdmsg, "binary    %s\n", exe_path::getExePath().c_str());
     fprintf(global.stdmsg, "version   %s (DMD %s, LLVM %s)\n",
             global.ldc_version, global.version, global.llvm_version);
-    const std::string &path = cfg_file.path();
+    const std::string path = cfg_file.path();
     if (!path.empty()) {
       fprintf(global.stdmsg, "config    %s (%s)\n", path.c_str(),
               cfg_triple.c_str());
@@ -409,13 +436,19 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   const auto toWinPaths = [](Strings *paths) {
     if (!paths)
       return;
-    for (unsigned i = 0; i < paths->dim; ++i)
-      (*paths)[i] = dupPathString((*paths)[i]);
+    for (auto &path : *paths)
+      path = dupPathString(path);
   };
   toWinPaths(global.params.imppath);
   toWinPaths(global.params.fileImppath);
 #endif
 
+#if LDC_LLVM_VER >= 400
+  if (saveOptimizationRecord.getNumOccurrences() > 0) {
+    global.params.outputSourceLocations = true;
+  }
+#endif
+
 // PGO options
 #if LDC_WITH_PGO
   if (genfileInstrProf.getNumOccurrences() > 0) {
@@ -438,6 +471,8 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
   }
 #endif
 
+  initializeSanitizerOptionsFromCmdline();
+
   processVersions(debugArgs, "debug", DebugCondition::setGlobalLevel,
                   DebugCondition::addGlobalIdent);
   processVersions(versions, "version", VersionCondition::setGlobalLevel,
@@ -445,6 +480,11 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
 
   processTransitions(transitions);
 
+  if (useDIP1000) {
+    global.params.useDIP25 = true;
+    global.params.vsafe = true;
+  }
+
   global.params.output_o =
       (opts::output_o == cl::BOU_UNSET &&
        !(opts::output_bc || opts::output_ll || opts::output_s))
@@ -528,29 +568,24 @@ void parseCommandLine(int argc, char **argv, Strings &sourceFiles,
 
   // LDC output determination
 
-  // if we don't link, autodetect target from extension
-  if (!global.params.link && !global.params.lib && global.params.objname) {
+  // if we don't link and there's no `-output-*` switch but an `-of` one,
+  // autodetect type of desired 'object' file from file extension
+  if (!global.params.link && !global.params.lib && global.params.objname &&
+      global.params.output_o == OUTPUTFLAGdefault) {
     const char *ext = FileName::ext(global.params.objname);
-    bool autofound = false;
     if (!ext) {
       // keep things as they are
-    } else if (strcmp(ext, global.ll_ext) == 0) {
+    } else if (opts::output_ll.getNumOccurrences() == 0 &&
+               strcmp(ext, global.ll_ext) == 0) {
       global.params.output_ll = OUTPUTFLAGset;
-      autofound = true;
-    } else if (strcmp(ext, global.bc_ext) == 0) {
+      global.params.output_o = OUTPUTFLAGno;
+    } else if (opts::output_bc.getNumOccurrences() == 0 &&
+               strcmp(ext, global.bc_ext) == 0) {
       global.params.output_bc = OUTPUTFLAGset;
-      autofound = true;
-    } else if (strcmp(ext, global.s_ext) == 0) {
+      global.params.output_o = OUTPUTFLAGno;
+    } else if (opts::output_s.getNumOccurrences() == 0 &&
+               strcmp(ext, global.s_ext) == 0) {
       global.params.output_s = OUTPUTFLAGset;
-      autofound = true;
-    } else if (strcmp(ext, global.obj_ext) == 0 || strcmp(ext, "obj") == 0) {
-      // global.obj_ext hasn't been corrected yet for MSVC targets as we first
-      // need the command line to figure out the target...
-      // so treat both 'o' and 'obj' extensions as object files
-      global.params.output_o = OUTPUTFLAGset;
-      autofound = true;
-    }
-    if (autofound && global.params.output_o == OUTPUTFLAGdefault) {
       global.params.output_o = OUTPUTFLAGno;
     }
   }
@@ -599,20 +634,15 @@ void initializePasses() {
   initializeTarget(Registry);
 
 // Initialize passes not included above
-#if LDC_LLVM_VER < 306
-  initializeDebugIRPass(Registry);
-#endif
 #if LDC_LLVM_VER < 308
   initializeIPA(Registry);
 #endif
 #if LDC_LLVM_VER >= 400
   initializeRewriteSymbolsLegacyPassPass(Registry);
-#elif LDC_LLVM_VER >= 306
+#else
   initializeRewriteSymbolsPass(Registry);
 #endif
-#if LDC_LLVM_VER >= 307
   initializeSjLjEHPreparePass(Registry);
-#endif
 }
 
 /// Register the MIPS ABI.
@@ -639,13 +669,9 @@ static void registerMipsABI() {
 /// Also defines D_HardFloat or D_SoftFloat depending if FPU should be used
 void registerPredefinedFloatABI(const char *soft, const char *hard,
                                 const char *softfp = nullptr) {
-// Use target floating point unit instead of s/w float routines
-#if LDC_LLVM_VER >= 307
+  // Use target floating point unit instead of s/w float routines
   // FIXME: This is a semantic change!
   bool useFPU = gTargetMachine->Options.FloatABIType == llvm::FloatABI::Hard;
-#else
-  bool useFPU = !gTargetMachine->Options.UseSoftFloat;
-#endif
   VersionCondition::addPredefinedGlobalIdent(useFPU ? "D_HardFloat"
                                                     : "D_SoftFloat");
 
@@ -663,7 +689,9 @@ void registerPredefinedFloatABI(const char *soft, const char *hard,
 /// Registers the predefined versions specific to the current target triple
 /// and other target specific options with VersionCondition.
 void registerPredefinedTargetVersions() {
-  switch (global.params.targetTriple->getArch()) {
+  const auto arch = global.params.targetTriple->getArch();
+
+  switch (arch) {
   case llvm::Triple::x86:
     VersionCondition::addPredefinedGlobalIdent("X86");
     if (global.params.useInlineAsm) {
@@ -705,10 +733,6 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent("ARM_Thumb");
     registerPredefinedFloatABI("ARM_SoftFloat", "ARM_HardFloat", "ARM_SoftFP");
     break;
-#if LDC_LLVM_VER == 305
-  case llvm::Triple::arm64:
-  case llvm::Triple::arm64_be:
-#endif
   case llvm::Triple::aarch64:
   case llvm::Triple::aarch64_be:
     VersionCondition::addPredefinedGlobalIdent("AArch64");
@@ -726,6 +750,9 @@ void registerPredefinedTargetVersions() {
     registerPredefinedFloatABI("MIPS_SoftFloat", "MIPS_HardFloat");
     registerMipsABI();
     break;
+  case llvm::Triple::msp430:
+    VersionCondition::addPredefinedGlobalIdent("MSP430");
+    break;
 #if defined RISCV_LLVM_DEV || LDC_LLVM_VER >= 400
 #if defined RISCV_LLVM_DEV
   case llvm::Triple::riscv:
@@ -774,15 +801,26 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent("BigEndian");
   }
 
-  // a generic 64bit version
+  // Set versions for arch bitwidth
   if (global.params.isLP64) {
     VersionCondition::addPredefinedGlobalIdent("D_LP64");
+  } else if (global.params.targetTriple->isArch16Bit()) {
+    VersionCondition::addPredefinedGlobalIdent("D_P16");
   }
 
   if (gTargetMachine->getRelocationModel() == llvm::Reloc::PIC_) {
     VersionCondition::addPredefinedGlobalIdent("D_PIC");
   }
 
+  /* LDC doesn't support DMD's core.simd interface.
+  if (arch == llvm::Triple::x86 || arch == llvm::Triple::x86_64) {
+    if (traitsTargetHasFeature("sse2"))
+      VersionCondition::addPredefinedGlobalIdent("D_SIMD");
+    if (traitsTargetHasFeature("avx"))
+      VersionCondition::addPredefinedGlobalIdent("D_AVX");
+  }
+  */
+
   // parse the OS out of the target triple
   // see http://gcc.gnu.org/install/specific.html for details
   // also llvm's different SubTargets have useful information
@@ -850,6 +888,10 @@ void registerPredefinedTargetVersions() {
     VersionCondition::addPredefinedGlobalIdent("AIX");
     VersionCondition::addPredefinedGlobalIdent("Posix");
     break;
+  case llvm::Triple::UnknownOS:
+    if (arch == llvm::Triple::msp430)
+      break;
+    // fallthrough
   default:
     switch (global.params.targetTriple->getEnvironment()) {
     case llvm::Triple::Android:
@@ -870,6 +912,12 @@ void registerPredefinedVersions() {
   VersionCondition::addPredefinedGlobalIdent("all");
   VersionCondition::addPredefinedGlobalIdent("D_Version2");
 
+#if LDC_LLVM_SUPPORTED_TARGET_SPIRV || LDC_LLVM_SUPPORTED_TARGET_NVPTX
+  if (dcomputeTargets.size() != 0) {
+    VersionCondition::addPredefinedGlobalIdent("LDC_DCompute");
+  }
+#endif
+
   if (global.params.doDocComments) {
     VersionCondition::addPredefinedGlobalIdent("D_Ddoc");
   }
@@ -896,16 +944,17 @@ void registerPredefinedVersions() {
     VersionCondition::addPredefinedGlobalIdent("D_ObjectiveC");
   }
 
-  // Pass sanitizer arguments to linker. Requires clang.
-  if (opts::sanitize == opts::AddressSanitizer) {
+  // Define sanitizer versions.
+  if (opts::isSanitizerEnabled(opts::AddressSanitizer)) {
     VersionCondition::addPredefinedGlobalIdent("LDC_AddressSanitizer");
   }
-
-  if (opts::sanitize == opts::MemorySanitizer) {
+  if (opts::isSanitizerEnabled(opts::CoverageSanitizer)) {
+    VersionCondition::addPredefinedGlobalIdent("LDC_CoverageSanitizer");
+  }
+  if (opts::isSanitizerEnabled(opts::MemorySanitizer)) {
     VersionCondition::addPredefinedGlobalIdent("LDC_MemorySanitizer");
   }
-
-  if (opts::sanitize == opts::ThreadSanitizer) {
+  if (opts::isSanitizerEnabled(opts::ThreadSanitizer)) {
     VersionCondition::addPredefinedGlobalIdent("LDC_ThreadSanitizer");
   }
 
@@ -917,24 +966,6 @@ void registerPredefinedVersions() {
 #undef STR
 }
 
-/// Dump all predefined version identifiers.
-void dumpPredefinedVersions() {
-  if (global.params.verbose && global.params.versionids) {
-    fprintf(global.stdmsg, "predefs  ");
-    int col = 10;
-    for (auto id : *global.params.versionids) {
-      int len = strlen(id) + 1;
-      if (col + len > 80) {
-        col = 10;
-        fprintf(global.stdmsg, "\n         ");
-      }
-      col += len;
-      fprintf(global.stdmsg, " %s", id);
-    }
-    fprintf(global.stdmsg, "\n");
-  }
-}
-
 } // anonymous namespace
 
 int cppmain(int argc, char **argv) {
@@ -965,7 +996,11 @@ int cppmain(int argc, char **argv) {
   Strings files;
   parseCommandLine(argc, argv, files, helpOnly);
 
-  if (files.dim == 0 && !helpOnly) {
+  if (helpOnly) {
+    return 0;
+  }
+
+  if (files.dim == 0) {
     cl::PrintHelpMessage();
     return EXIT_FAILURE;
   }
@@ -997,10 +1032,6 @@ int cppmain(int argc, char **argv) {
 #if LDC_LLVM_VER >= 308
   static llvm::DataLayout DL = gTargetMachine->createDataLayout();
   gDataLayout = &DL;
-#elif LDC_LLVM_VER >= 307
-  gDataLayout = gTargetMachine->getDataLayout();
-#elif LDC_LLVM_VER >= 306
-  gDataLayout = gTargetMachine->getSubtargetImpl()->getDataLayout();
 #else
   gDataLayout = gTargetMachine->getDataLayout();
 #endif
@@ -1038,14 +1069,15 @@ int cppmain(int argc, char **argv) {
 
 void addDefaultVersionIdentifiers() {
   registerPredefinedVersions();
-  dumpPredefinedVersions();
+  printPredefinedVersions();
 }
 
 void codegenModules(Modules &modules) {
-  // Generate one or more object/IR/bitcode files.
+  // Generate one or more object/IR/bitcode files/dcompute kernels.
   if (global.params.obj && !modules.empty()) {
     ldc::CodeGenerator cg(getGlobalContext(), global.params.oneobj);
-
+    DComputeCodeGenManager dccg(getGlobalContext());
+    std::vector<Module *> computeModules;
     // When inlining is enabled, we are calling semantic3 on function
     // declarations, which may _add_ members to the first module in the modules
     // array. These added functions must be codegenned, because these functions
@@ -1059,11 +1091,39 @@ void codegenModules(Modules &modules) {
       if (global.params.verbose)
         fprintf(global.stdmsg, "code      %s\n", m->toChars());
 
-      cg.emit(m);
-
+      const auto atCompute = hasComputeAttr(m);
+      if (atCompute == DComputeCompileFor::hostOnly ||
+           atCompute == DComputeCompileFor::hostAndDevice)
+      {
+        cg.emit(m);
+      }
+      if (atCompute != DComputeCompileFor::hostOnly) {
+        computeModules.push_back(m);
+        if (atCompute == DComputeCompileFor::deviceOnly) {
+          // Remove m's object file from list of object files
+          auto s = m->objfile->name->str;
+          for (size_t j = 0; j < global.params.objfiles->dim; j++) {
+            if (s == (*global.params.objfiles)[j]) {
+              global.params.objfiles->remove(j);
+              break;
+            }
+          }
+        }
+      }
       if (global.errors)
         fatal();
     }
+
+    if (!computeModules.empty()) {
+      for (auto& mod : computeModules)
+        dccg.emit(mod);
+
+      dccg.writeModules();
+    }
+    // We may have removed all object files, if so don't link.
+    if (global.params.objfiles->dim == 0)
+      global.params.link = false;
+
   }
 
   cache::pruneCache();
