@@ -30,86 +30,6 @@
 
 //////////////////////////////////////////////////////////////////////////////
 
-namespace {
-// from dmd/src/typinf.c
-bool isSpeculativeType(Type *t) {
-  class SpeculativeTypeVisitor : public Visitor {
-  public:
-    bool result;
-
-    SpeculativeTypeVisitor() : result(false) {}
-
-    using Visitor::visit;
-    void visit(Type *t) override {
-      Type *tb = t->toBasetype();
-      if (tb != t) {
-        tb->accept(this);
-      }
-    }
-    void visit(TypeNext *t) override {
-      if (t->next) {
-        t->next->accept(this);
-      }
-    }
-    void visit(TypeBasic *t) override {}
-    void visit(TypeVector *t) override { t->basetype->accept(this); }
-    void visit(TypeAArray *t) override {
-      t->index->accept(this);
-      visit((TypeNext *)t);
-    }
-    void visit(TypeFunction *t) override {
-      visit((TypeNext *)t);
-      // Currently TypeInfo_Function doesn't store parameter types.
-    }
-    void visit(TypeStruct *t) override {
-      StructDeclaration *sd = t->sym;
-      if (TemplateInstance *ti = sd->isInstantiated()) {
-        if (!ti->needsCodegen()) {
-          if (ti->minst || sd->requestTypeInfo) {
-            return;
-          }
-
-          /* Bugzilla 14425: TypeInfo_Struct would refer the members of
-           * struct (e.g. opEquals via xopEquals field), so if it's instantiated
-           * in speculative context, TypeInfo creation should also be
-           * stopped to avoid 'unresolved symbol' linker errors.
-           */
-          /* When -debug/-unittest is specified, all of non-root instances are
-           * automatically changed to speculative, and here is always reached
-           * from those instantiated non-root structs.
-           * Therefore, if the TypeInfo is not auctually requested,
-           * we have to elide its codegen.
-           */
-          result |= true;
-          return;
-        }
-      } else {
-        // assert(!sd->inNonRoot() || sd->requestTypeInfo);  // valid?
-      }
-    }
-    void visit(TypeClass *t) override {}
-    void visit(TypeTuple *t) override {
-      if (t->arguments) {
-        for (size_t i = 0; i < t->arguments->dim; i++) {
-          Type *tprm = (*t->arguments)[i]->type;
-          if (tprm) {
-            tprm->accept(this);
-          }
-          if (result) {
-            return;
-          }
-        }
-      }
-    }
-  };
-  SpeculativeTypeVisitor v;
-  t->accept(&v);
-  return v.result;
-}
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 class CodegenVisitor : public Visitor {
   IRState *irs;
 
@@ -147,6 +67,8 @@ public:
                            decl->toPrettyChars());
     LOG_SCOPE
 
+    assert(!irs->dcomputetarget);
+
     if (decl->ir->isDefined()) {
       return;
     }
@@ -205,14 +127,20 @@ public:
       m->accept(this);
     }
 
-    // Define the __initZ symbol.
-    IrAggr *ir = getIrAggr(decl);
-    llvm::GlobalVariable *initZ = ir->getInitSymbol();
-    initZ->setInitializer(ir->getDefaultInit());
-    setLinkage(decl, initZ);
+    // Skip __initZ and typeinfo for @compute device code.
+    // TODO: support global variables and thus __initZ
+    if (!irs->dcomputetarget) {
+      // Define the __initZ symbol.
+      IrAggr *ir = getIrAggr(decl);
+      auto &initZ = ir->getInitSymbol();
+      auto initGlobal = llvm::cast<LLGlobalVariable>(initZ);
+      assert(initGlobal);
+      setLinkage(decl, initGlobal);
+      initZ = irs->setGlobalVarInitializer(initGlobal, ir->getDefaultInit());
 
-    // emit typeinfo
-    DtoTypeInfoOf(decl->type);
+      // emit typeinfo
+      DtoTypeInfoOf(decl->type);
+    }
 
     // Emit __xopEquals/__xopCmp/__xtoHash.
     if (decl->xeq && decl->xeq != decl->xerreq) {
@@ -233,6 +161,8 @@ public:
                            decl->toPrettyChars());
     LOG_SCOPE
 
+    assert(!irs->dcomputetarget);
+
     if (decl->ir->isDefined()) {
       return;
     }
@@ -254,9 +184,11 @@ public:
       IrAggr *ir = getIrAggr(decl);
       const auto lwc = DtoLinkage(decl);
 
-      llvm::GlobalVariable *initZ = ir->getInitSymbol();
-      initZ->setInitializer(ir->getDefaultInit());
-      setLinkage(lwc, initZ);
+      auto &initZ = ir->getInitSymbol();
+      auto initGlobal = llvm::cast<LLGlobalVariable>(initZ);
+      assert(initGlobal);
+      setLinkage(lwc, initGlobal);
+      initZ = irs->setGlobalVarInitializer(initGlobal, ir->getDefaultInit());
 
       llvm::GlobalVariable *vtbl = ir->getVtblSymbol();
       vtbl->setInitializer(ir->getVtblInit());
@@ -325,6 +257,7 @@ public:
 
       assert(!(decl->storage_class & STCmanifest) &&
              "manifest constant being codegen'd!");
+      assert(!irs->dcomputetarget);
 
       IrGlobal *irGlobal = getIrGlobal(decl);
       LLGlobalVariable *gvar = llvm::cast<LLGlobalVariable>(irGlobal->value);
@@ -337,45 +270,26 @@ public:
         const char *p = decl->loc.toChars();
         fprintf(global.stdmsg, "%s: %s is thread local\n", p, decl->toChars());
       }
+
       // Check if we are defining or just declaring the global in this module.
       // If we reach here during codegen of an available_externally function,
       // new variable declarations should stay external and therefore must not
       // have an initializer.
       if (!(decl->storage_class & STCextern) && !decl->inNonRoot()) {
-        // Build the initializer. Might use this->ir.irGlobal->value!
+        assert(!gvar->hasDLLImportStorageClass());
+
+        // Build the initializer. Might use irGlobal->value!
         LLConstant *initVal =
             DtoConstInitializer(decl->loc, decl->type, decl->_init);
 
-        // In case of type mismatch, swap out the variable.
-        if (initVal->getType() != gvar->getType()->getElementType()) {
-          llvm::GlobalVariable *newGvar = getOrCreateGlobal(
-              decl->loc, irs->module, initVal->getType(), gvar->isConstant(),
-              lwc.first, nullptr,
-              "", // We take on the name of the old global below.
-              gvar->isThreadLocal());
-          setLinkage(lwc, newGvar);
-
-          newGvar->setAlignment(gvar->getAlignment());
-          newGvar->setDLLStorageClass(gvar->getDLLStorageClass());
-          applyVarDeclUDAs(decl, newGvar);
-          newGvar->takeName(gvar);
-
-          llvm::Constant *newValue =
-              llvm::ConstantExpr::getBitCast(newGvar, gvar->getType());
-          gvar->replaceAllUsesWith(newValue);
-
-          gvar->eraseFromParent();
-          gvar = newGvar;
-          irGlobal->value = newGvar;
-        }
-
-        assert(!gvar->hasDLLImportStorageClass());
-
-        // Now, set the initializer.
+        // Cache it.
         assert(!irGlobal->constInit);
         irGlobal->constInit = initVal;
-        gvar->setInitializer(initVal);
+
+        // Set the initializer, swapping out the variable if the types do not
+        // match.
         setLinkage(lwc, gvar);
+        irGlobal->value = irs->setGlobalVarInitializer(gvar, initVal);
 
         // Also set up the debug info.
         irs->DBuilder.EmitGlobalVariable(gvar, decl);
@@ -492,7 +406,8 @@ public:
   void visit(PragmaDeclaration *decl) LLVM_OVERRIDE {
     if (decl->ident == Id::lib) {
       assert(decl->args && decl->args->dim == 1);
-
+      assert(!irs->dcomputetarget);
+        
       Expression *e = static_cast<Expression *>(decl->args->data[0]);
 
       assert(e->op == TOKstring);
@@ -553,7 +468,7 @@ public:
   //////////////////////////////////////////////////////////////////////////
 
   void visit(TypeInfoDeclaration *decl) LLVM_OVERRIDE {
-    if (isSpeculativeType(decl->tinfo)) {
+    if (irs->dcomputetarget || isSpeculativeType(decl->tinfo)) {
       return;
     }
     TypeInfoDeclaration_codegen(decl, irs);
@@ -562,7 +477,7 @@ public:
   //////////////////////////////////////////////////////////////////////////
 
   void visit(TypeInfoClassDeclaration *decl) LLVM_OVERRIDE {
-    if (isSpeculativeType(decl->tinfo)) {
+    if (irs->dcomputetarget || isSpeculativeType(decl->tinfo)) {
       return;
     }
     TypeInfoClassDeclaration_codegen(decl, irs);
@@ -572,8 +487,7 @@ public:
 //////////////////////////////////////////////////////////////////////////////
 
 void Declaration_codegen(Dsymbol *decl) {
-  CodegenVisitor v(gIR);
-  decl->accept(&v);
+  Declaration_codegen(decl, gIR);
 }
 
 void Declaration_codegen(Dsymbol *decl, IRState *irs) {
