@@ -9,6 +9,7 @@
 #include "gen/irstate.h"
 #include "gen/logger.h"
 #include "gen/tollvm.h"
+#include "gen/to_string.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/Support/SourceMgr.h"
 #include "llvm/AsmParser/Parser.h"
@@ -35,6 +36,7 @@ struct TempDisableDiscardValueNames {
 };
 
 /// Adds the idol's function attributes to the wannabe
+/// Note: don't add function _parameter_ attributes
 void copyFnAttributes(llvm::Function *wannabe, llvm::Function *idol) {
   auto attrSet = idol->getAttributes();
   auto fnAttrSet = attrSet.getFnAttributes();
@@ -55,7 +57,7 @@ DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
   // always inlined, this name does not escape the current compiled module; not
   // even at -O0.
   static size_t namecounter = 0;
-  std::string mangled_name = "inline.ir." + std::to_string(namecounter++);
+  std::string mangled_name = "inline.ir." + ldc::to_string(namecounter++);
   TemplateInstance *tinst = fdecl->parent->isTemplateInstance();
   assert(tinst);
 
@@ -110,13 +112,8 @@ DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
 
     llvm::SMDiagnostic err;
 
-#if LDC_LLVM_VER >= 306
     std::unique_ptr<llvm::Module> m =
         llvm::parseAssemblyString(stream.str().c_str(), err, gIR->context());
-#else
-    llvm::Module *m = llvm::ParseAssemblyString(stream.str().c_str(), NULL, err,
-                                                gIR->context());
-#endif
 
     std::string errstr = err.getMessage();
     if (!errstr.empty()) {
@@ -132,15 +129,8 @@ DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
 
 #if LDC_LLVM_VER >= 308
     llvm::Linker(gIR->module).linkInModule(std::move(m));
-#elif LDC_LLVM_VER >= 306
-    llvm::Linker(&gIR->module).linkInModule(m.get());
 #else
-    errstr.clear();
-    llvm::Linker(&gIR->module).linkInModule(m, &errstr);
-    if (!errstr.empty()) {
-      error(tinst->loc, "Error when linking in llvm inline ir: %s",
-            errstr.c_str());
-    }
+    llvm::Linker(&gIR->module).linkInModule(m.get());
 #endif
   }
 
@@ -164,11 +154,10 @@ DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
     fun->setCallingConv(llvm::CallingConv::C);
 
     // Build the runtime arguments
-    size_t n = arguments->dim;
     llvm::SmallVector<llvm::Value *, 8> args;
-    args.reserve(n);
-    for (size_t i = 0; i < n; i++) {
-      args.push_back(DtoRVal((*arguments)[i]));
+    args.reserve(arguments->dim);
+    for (auto arg : *arguments) {
+      args.push_back(DtoRVal(arg));
     }
 
     llvm::Value *rv = gIR->ir->CreateCall(fun, args);
