@@ -38,6 +38,7 @@ class IndexedInstrProfReader;
 class FuncGenState;
 struct IRState;
 struct TargetABI;
+class DComputeTarget;
 
 extern IRState *gIR;
 extern llvm::TargetMachine *gTargetMachine;
@@ -106,6 +107,11 @@ struct IRAsmBlock {
 
 // represents the module
 struct IRState {
+private:
+  std::vector<std::pair<llvm::GlobalVariable *, llvm::Constant *>>
+      globalsToReplace;
+
+public:
   IRState(const char *name, llvm::LLVMContext &context);
   ~IRState();
 
@@ -198,11 +204,26 @@ struct IRState {
   llvm::StringMap<llvm::GlobalVariable *> stringLiteral2ByteCache;
   llvm::StringMap<llvm::GlobalVariable *> stringLiteral4ByteCache;
 
+  // Sets the initializer for a global LL variable.
+  // If the types don't match, this entails creating a new helper global
+  // matching the initializer type and replacing all existing uses of globalVar
+  // by a bitcast pointer to the helper global's payload.
+  // Returns either the specified globalVar if the types match, or the bitcast
+  // pointer replacing globalVar (and resets globalVar to the new helper
+  // global).
+  llvm::Constant *setGlobalVarInitializer(llvm::GlobalVariable *&globalVar,
+                                          llvm::Constant *initializer);
+
+  // To be called when finalizing the IR module in order to perform a second
+  // replacement pass for global variables replaced (and registered) by
+  // setGlobalVarInitializer().
+  void replaceGlobals();
+
 /// Vector of options passed to the linker as metadata in object file.
-#if LDC_LLVM_VER >= 306
-  llvm::SmallVector<llvm::Metadata *, 5> LinkerMetadataArgs;
+#if LDC_LLVM_VER >= 500
+  llvm::SmallVector<llvm::MDNode *, 5> LinkerMetadataArgs;
 #else
-  llvm::SmallVector<llvm::Value *, 5> LinkerMetadataArgs;
+  llvm::SmallVector<llvm::Metadata *, 5> LinkerMetadataArgs;
 #endif
 
 #if LDC_LLVM_VER >= 308
@@ -210,6 +231,9 @@ struct IRState {
   llvm::DenseMap<size_t, llvm::StructType *> TypeDescriptorTypeMap;
   llvm::DenseMap<llvm::Constant *, llvm::GlobalVariable *> TypeDescriptorMap;
 #endif
+
+  //Target for dcompute. If not nullptr, it owns this.
+  DComputeTarget *dcomputetarget;
 };
 
 void Statement_toIR(Statement *s, IRState *irs);
