@@ -46,7 +46,7 @@
 #include "llvm/Support/CommandLine.h"
 
 llvm::cl::opt<llvm::GlobalVariable::ThreadLocalMode> clThreadModel(
-    "fthread-model", llvm::cl::desc("Thread model"),
+    "fthread-model", llvm::cl::ZeroOrMore, llvm::cl::desc("Thread model"),
     llvm::cl::init(llvm::GlobalVariable::GeneralDynamicTLSModel),
     clEnumValues(clEnumValN(llvm::GlobalVariable::GeneralDynamicTLSModel,
                             "global-dynamic",
@@ -71,8 +71,8 @@ bool isTargetWindowsMSVC() {
 }
 
 /******************************************************************************
-* Global context
-******************************************************************************/
+ * Global context
+ ******************************************************************************/
 static llvm::ManagedStatic<llvm::LLVMContext> GlobalContext;
 
 llvm::LLVMContext &getGlobalContext() { return *GlobalContext; }
@@ -180,15 +180,24 @@ llvm::AllocaInst *DtoAlloca(VarDeclaration *vd, const char *name) {
 llvm::AllocaInst *DtoArrayAlloca(Type *type, unsigned arraysize,
                                  const char *name) {
   LLType *lltype = DtoType(type);
-  auto ai = new llvm::AllocaInst(lltype, DtoConstUint(arraysize), name,
-                                 gIR->topallocapoint());
+  auto ai = new llvm::AllocaInst(
+      lltype,
+#if LDC_LLVM_VER >= 500
+      gIR->module.getDataLayout().getAllocaAddrSpace(),
+#endif
+      DtoConstUint(arraysize), name, gIR->topallocapoint());
   ai->setAlignment(DtoAlignment(type));
   return ai;
 }
 
 llvm::AllocaInst *DtoRawAlloca(LLType *lltype, size_t alignment,
                                const char *name) {
-  auto ai = new llvm::AllocaInst(lltype, name, gIR->topallocapoint());
+  auto ai =
+      new llvm::AllocaInst(lltype,
+#if LDC_LLVM_VER >= 500
+                           gIR->module.getDataLayout().getAllocaAddrSpace(),
+#endif
+                           name, gIR->topallocapoint());
   if (alignment) {
     ai->setAlignment(alignment);
   }
@@ -306,8 +315,6 @@ void DtoAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
   LOG_SCOPE;
 
   Type *t = lhs->type->toBasetype();
-  Type *t2 = rhs->type->toBasetype();
-
   assert(t->ty != Tvoid && "Cannot assign values of type void.");
 
   if (t->ty == Tbool) {
@@ -335,7 +342,7 @@ void DtoAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
     }
     DtoStore(r, l);
   } else if (t->ty == Tclass) {
-    assert(t2->ty == Tclass);
+    assert(rhs->type->toBasetype()->ty == Tclass);
     LLValue *l = DtoLVal(lhs);
     LLValue *r = DtoRVal(rhs);
     IF_LOG {
@@ -469,6 +476,7 @@ DValue *DtoCastPtr(Loc &loc, DValue *val, Type *to) {
 
   Type *totype = to->toBasetype();
   Type *fromtype = val->type->toBasetype();
+  (void)fromtype;
   assert(fromtype->ty == Tpointer || fromtype->ty == Tfunction);
 
   LLValue *rval;
@@ -980,8 +988,7 @@ DValue *DtoDeclarationExp(Dsymbol *declaration) {
   } else if (AttribDeclaration *a = declaration->isAttribDeclaration()) {
     Logger::println("AttribDeclaration");
     // choose the right set in case this is a conditional declaration
-    Dsymbols *d = a->include(nullptr, nullptr);
-    if (d) {
+    if (auto d = a->include(nullptr, nullptr)) {
       for (unsigned i = 0; i < d->dim; ++i) {
         DtoDeclarationExp((*d)[i]);
       }
@@ -996,7 +1003,7 @@ DValue *DtoDeclarationExp(Dsymbol *declaration) {
     assert(tupled->isexp && "Non-expression tuple decls not handled yet.");
     assert(tupled->objects);
     for (unsigned i = 0; i < tupled->objects->dim; ++i) {
-      DsymbolExp *exp = static_cast<DsymbolExp *>(tupled->objects->data[i]);
+      auto exp = static_cast<DsymbolExp *>((*tupled->objects)[i]);
       DtoDeclarationExp(exp->s);
     }
   } else {
@@ -1162,6 +1169,7 @@ LLConstant *DtoConstExpInit(Loc &loc, Type *targetType, Expression *exp) {
       val = llvm::ConstantArray::get(at, elements);
     }
 
+    (void)numTotalVals;
     assert(product == numTotalVals);
     return val;
   }
@@ -1183,9 +1191,11 @@ LLConstant *DtoConstExpInit(Loc &loc, Type *targetType, Expression *exp) {
     llvm::IntegerType *source = llvm::cast<llvm::IntegerType>(llType);
     llvm::IntegerType *target = llvm::cast<llvm::IntegerType>(targetLLType);
 
+    (void)source;
     assert(target->getBitWidth() > source->getBitWidth() &&
            "On initializer integer type mismatch, the target should be wider "
            "than the source.");
+
     return llvm::ConstantExpr::getZExtOrBitCast(val, target);
   }
 
@@ -1291,42 +1301,6 @@ void DtoSetFuncDeclIntrinsicName(TemplateInstance *ti, TemplateDeclaration *td,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-bool hasUnalignedFields(Type *t) {
-  t = t->toBasetype();
-  if (t->ty == Tsarray) {
-    assert(t->nextOf()->size() % t->nextOf()->alignsize() == 0);
-    return hasUnalignedFields(t->nextOf());
-  }
-  if (t->ty != Tstruct) {
-    return false;
-  }
-
-  TypeStruct *ts = static_cast<TypeStruct *>(t);
-  if (ts->unaligned) {
-    return (ts->unaligned == 2);
-  }
-
-  StructDeclaration *sym = ts->sym;
-
-  // go through all the fields and try to find something unaligned
-  ts->unaligned = 2;
-  for (unsigned i = 0; i < sym->fields.dim; i++) {
-    VarDeclaration *f = static_cast<VarDeclaration *>(sym->fields.data[i]);
-    unsigned a = f->type->alignsize() - 1;
-    if (((f->offset + a) & ~a) != f->offset) {
-      return true;
-    }
-    if (f->type->toBasetype()->ty == Tstruct && hasUnalignedFields(f->type)) {
-      return true;
-    }
-  }
-
-  ts->unaligned = 1;
-  return false;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-
 size_t getMemberSize(Type *type) {
   const dinteger_t dSize = type->size();
   llvm::Type *const llType = DtoType(type);
@@ -1407,9 +1381,10 @@ bool isLLVMUnsigned(Type *t) { return t->isunsigned() || t->ty == Tpointer; }
 
 void printLabelName(std::ostream &target, const char *func_mangle,
                     const char *label_name) {
-  target << gTargetMachine->getMCAsmInfo()->getPrivateGlobalPrefix()
+  target << gTargetMachine->getMCAsmInfo()
+                ->getPrivateGlobalPrefix()
 #if LDC_LLVM_VER >= 400
-              .str()
+                .str()
 #endif
          << func_mangle << "_" << label_name;
 }
@@ -1610,9 +1585,9 @@ DValue *DtoSymbolAddress(Loc &loc, Type *type, Declaration *decl) {
       fatal();
     }
     DtoResolveFunction(fdecl);
-    return new DFuncValue(fdecl, fdecl->llvmInternal != LLVMva_arg
-                                     ? DtoCallee(fdecl)
-                                     : nullptr);
+    const auto llValue =
+        fdecl->llvmInternal != LLVMva_arg ? DtoCallee(fdecl) : nullptr;
+    return new DFuncValue(fdecl, llValue);
   }
 
   if (SymbolDeclaration *sdecl = decl->isSymbolDeclaration()) {
@@ -1648,8 +1623,9 @@ llvm::Constant *DtoConstSymbolAddress(Loc &loc, Declaration *decl) {
       // needed for the current hacky implementation of
       // AssocArrayLiteralExp::toElem, which requires on error
       // gagging to check for constantness of the initializer.
-      error(loc, "cannot use address of non-global variable '%s' "
-                 "as constant initializer",
+      error(loc,
+            "cannot use address of non-global variable '%s' as constant "
+            "initializer",
             vd->toChars());
       if (!global.gag) {
         fatal();
@@ -1716,8 +1692,9 @@ llvm::GlobalVariable *getOrCreateGlobal(const Loc &loc, llvm::Module &module,
   llvm::GlobalVariable *existing = module.getGlobalVariable(name, true);
   if (existing) {
     if (existing->getType()->getElementType() != type) {
-      error(loc, "Global variable type does not match previous "
-                 "declaration with same mangled name: %s",
+      error(loc,
+            "Global variable type does not match previous declaration with "
+            "same mangled name: %s",
             name.str().c_str());
       fatal();
     }
