@@ -31,7 +31,6 @@
 #include "gen/logger.h"
 #include "gen/moduleinfo.h"
 #include "gen/optimizer.h"
-#include "gen/programs.h"
 #include "gen/runtime.h"
 #include "gen/structs.h"
 #include "gen/tollvm.h"
@@ -55,14 +54,13 @@
 #endif
 
 static llvm::cl::opt<bool, true>
-    preservePaths("op", llvm::cl::desc("Preserve source path for output files"),
-                  llvm::cl::ZeroOrMore,
+    preservePaths("op", llvm::cl::ZeroOrMore,
+                  llvm::cl::desc("Preserve source path for output files"),
                   llvm::cl::location(global.params.preservePaths));
 
 static llvm::cl::opt<bool, true>
-    fqnNames("oq",
+    fqnNames("oq", llvm::cl::ZeroOrMore,
              llvm::cl::desc("Write object files with fully qualified names"),
-             llvm::cl::ZeroOrMore,
              llvm::cl::location(global.params.fullyQualifiedObjectFiles));
 
 void Module::checkAndAddOutputFile(File *file) {
@@ -72,8 +70,9 @@ void Module::checkAndAddOutputFile(File *file) {
   auto i = files.find(key);
   if (i != files.end()) {
     Module *previousMod = i->second;
-    ::error(Loc(), "Output file '%s' for module '%s' collides with previous "
-                   "module '%s'. See the -oq option",
+    ::error(Loc(),
+            "Output file '%s' for module '%s' collides with previous "
+            "module '%s'. See the -oq option",
             key.c_str(), toPrettyChars(), previousMod->toPrettyChars());
     fatal();
   }
@@ -121,16 +120,8 @@ RegistryStyle getModuleRegistryStyle() {
     return RegistryStyle::sectionDarwin;
   }
 
-  if ((t->isOSLinux() && t->getEnvironment() != llvm::Triple::Android) ||
-      t->isOSFreeBSD() ||
-#if LDC_LLVM_VER > 305
-      t->isOSNetBSD() || t->isOSOpenBSD() || t->isOSDragonFly()
-#else
-      t->getOS() == llvm::Triple::NetBSD ||
-      t->getOS() == llvm::Triple::OpenBSD ||
-      t->getOS() == llvm::Triple::DragonFly
-#endif
-          ) {
+  if (t->isOSLinux() || t->isOSFreeBSD() ||
+      t->isOSNetBSD() || t->isOSOpenBSD() || t->isOSDragonFly()) {
     return RegistryStyle::sectionELF;
   }
 
@@ -198,10 +189,7 @@ LLFunction *build_module_reference_and_ctor(const char *moduleMangle,
 
   // put current beginning as the next of this one
   LLValue *gep = builder.CreateStructGEP(
-#if LDC_LLVM_VER >= 307
-      modulerefTy,
-#endif
-      thismref, 0, "next");
+      modulerefTy, thismref, 0, "next");
   builder.CreateStore(curbeg, gep);
 
   // replace beginning
@@ -281,7 +269,7 @@ llvm::Function *buildRegisterDSO(RegistryStyle style,
       getRuntimeFunction(Loc(), gIR->module, "_d_dso_registry");
   const auto recordPtrTy = dsoRegistry->getFunctionType()->getContainedType(1);
 
-  llvm::Function *getTlsAnchorPtr;
+  llvm::Function *getTlsAnchorPtr = nullptr;
   if (style == RegistryStyle::sectionDarwin) {
     getTlsAnchorPtr = buildGetTLSAnchor();
   }
@@ -292,7 +280,8 @@ llvm::Function *buildRegisterDSO(RegistryStyle style,
 
   {
     IRBuilder<> b(entryBB);
-    const auto loadedFlag = b.CreateTrunc(b.CreateLoad(dsoInitialized), b.getInt1Ty());
+    const auto loadedFlag =
+        b.CreateTrunc(b.CreateLoad(dsoInitialized), b.getInt1Ty());
     const auto condEval =
         b.CreateICmp(llvm::ICmpInst::ICMP_EQ, loadedFlag, isShutdown);
     b.CreateCondBr(condEval, initBB, endBB);
@@ -317,7 +306,6 @@ llvm::Function *buildRegisterDSO(RegistryStyle style,
     llvm::Value *record = b.CreateAlloca(stype);
 
     unsigned i = 0;
-#if LDC_LLVM_VER >= 307
     b.CreateStore(version, b.CreateStructGEP(stype, record, i++));
     b.CreateStore(dsoSlot, b.CreateStructGEP(stype, record, i++));
     b.CreateStore(minfoBeg, b.CreateStructGEP(stype, record, i++));
@@ -326,16 +314,6 @@ llvm::Function *buildRegisterDSO(RegistryStyle style,
       b.CreateStore(getTlsAnchorPtr, b.CreateStructGEP(stype, record, i++));
     }
     b.CreateStore(minfoUsedPointer, b.CreateStructGEP(stype, record, i++));
-#else
-    b.CreateStore(version, b.CreateStructGEP(record, i++));
-    b.CreateStore(dsoSlot, b.CreateStructGEP(record, i++));
-    b.CreateStore(minfoBeg, b.CreateStructGEP(record, i++));
-    b.CreateStore(minfoEnd, b.CreateStructGEP(record, i++));
-    if (style == RegistryStyle::sectionDarwin) {
-      b.CreateStore(getTlsAnchorPtr, b.CreateStructGEP(record, i++));
-    }
-    b.CreateStore(minfoUsedPointer, b.CreateStructGEP(record, i++));
-#endif
 
     b.CreateCall(dsoRegistry, b.CreateBitCast(record, recordPtrTy));
     b.CreateBr(endBB);
@@ -376,7 +354,10 @@ void emitModuleRefToSection(RegistryStyle style, std::string moduleMangle,
                                                                : "__minfo");
   gIR->usedArray.push_back(thismref);
 
-  if (!isFirst) {
+  // Android doesn't need register_dso and friends- see rt.sections_android-
+  // so bail out here.
+  if (!isFirst ||
+      global.params.targetTriple->getEnvironment() == llvm::Triple::Android) {
     // Nothing left to do.
     return;
   }
@@ -518,10 +499,7 @@ void addCoverageAnalysis(Module *m) {
     d_cover_valid_slice =
         DtoConstSlice(DtoConstSize_t(type->getArrayNumElements()),
                       llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
-                          type,
-#endif
-                          m->d_cover_valid, idxs, true));
+                          type, m->d_cover_valid, idxs, true));
 
     // Assert that initializer array elements have enough bits
     assert(sizeof(m->d_cover_valid_init[0]) * 8 >=
@@ -547,10 +525,7 @@ void addCoverageAnalysis(Module *m) {
     d_cover_data_slice =
         DtoConstSlice(DtoConstSize_t(type->getArrayNumElements()),
                       llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
-                          type,
-#endif
-                          m->d_cover_data, idxs, true));
+                          type, m->d_cover_data, idxs, true));
   }
 
   // Create "static constructor" that calls _d_cover_register2(string filename,
@@ -689,8 +664,7 @@ void registerModuleInfo(Module *m) {
   const char *mangle = mangleBuf.peekString();
 
   if (style == RegistryStyle::legacyLinkedList) {
-    const auto miCtor =
-        build_module_reference_and_ctor(mangle, moduleInfoSym);
+    const auto miCtor = build_module_reference_and_ctor(mangle, moduleInfoSym);
     AppendFunctionToLLVMGlobalCtorsDtors(miCtor, 65535, true);
   } else {
     emitModuleRefToSection(style, mangle, moduleInfoSym);
@@ -719,6 +693,7 @@ void codegenModule(IRState *irs, Module *m) {
   }
 
   // process module members
+  // NOTE: m->members may grow during codegen
   for (unsigned k = 0; k < m->members->dim; k++) {
     Dsymbol *dsym = (*m->members)[k];
     assert(dsym);
