@@ -37,9 +37,9 @@
 #if LDC_LLVM_VER >= 309
 namespace {
 llvm::cl::opt<bool, false, opts::FlagParser<bool>> enablePGOIndirectCalls(
-    "pgo-indirect-calls",
+    "pgo-indirect-calls", llvm::cl::ZeroOrMore, llvm::cl::Hidden,
     llvm::cl::desc("(*) Enable PGO of indirect calls (LLVM >= 3.9)"),
-    llvm::cl::init(true), llvm::cl::Hidden);
+    llvm::cl::init(true));
 }
 #endif
 
@@ -184,6 +184,7 @@ struct MapRegionCounters : public StoppableVisitor {
   void visit(Expression *exp) override {}
   void visit(Declaration *decl) override {}
   void visit(Initializer *init) override {}
+  void visit(Dsymbol *dsym) override {}
 
   void visit(FuncDeclaration *fd) override {
     if (NextCounter) {
@@ -928,15 +929,17 @@ void CodeGenPGO::loadRegionCounts(llvm::IndexedInstrProfReader *PGOReader,
       IF_LOG Logger::println(
           "Ignoring profile data: hash mismatch for function: %s",
           FuncName.c_str());
-      warning(fd->loc, "Ignoring profile data for function '%s' ('%s'): "
-                       "control-flow hash mismatch",
+      warning(fd->loc,
+              "Ignoring profile data for function '%s' ('%s'): "
+              "control-flow hash mismatch",
               const_cast<FuncDeclaration *>(fd)->toPrettyChars(),
               FuncName.c_str());
     } else if (IPE == llvm::instrprof_error::malformed) {
       IF_LOG Logger::println("Profile data is malformed for function: %s",
                              FuncName.c_str());
-      warning(fd->loc, "Ignoring profile data for function '%s' ('%s'): "
-                       "control-flow hash mismatch",
+      warning(fd->loc,
+              "Ignoring profile data for function '%s' ('%s'): "
+              "control-flow hash mismatch",
               const_cast<FuncDeclaration *>(fd)->toPrettyChars(),
               FuncName.c_str());
     } else {
