@@ -43,10 +43,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 static llvm::cl::opt<bool> nogc(
-    "nogc",
+    "nogc", llvm::cl::ZeroOrMore,
     llvm::cl::desc(
-        "Do not allow code that generates implicit garbage collector calls"),
-    llvm::cl::ZeroOrMore);
+        "Do not allow code that generates implicit garbage collector calls"));
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -97,8 +96,9 @@ static void checkForImplicitGCCall(const Loc &loc, const char *name) {
 
     if (binary_search(&GCNAMES[0],
                       &GCNAMES[sizeof(GCNAMES) / sizeof(std::string)], name)) {
-      error(loc, "No implicit garbage collector calls allowed with -nogc "
-                 "option enabled: %s",
+      error(loc,
+            "No implicit garbage collector calls allowed with -nogc "
+            "option enabled: %s",
             name);
       fatal();
     }
@@ -218,7 +218,7 @@ template <typename DECL> static void ensureDecl(DECL *decl, const char *msg) {
     Logger::println("Missing class declaration: %s\n", msg);
     error(Loc(), "Missing class declaration: %s", msg);
     errorSupplemental(Loc(),
-                      "Please check that object.di is included and valid");
+                      "Please check that object.d is included and valid");
     fatal();
   }
 }
@@ -289,7 +289,7 @@ static void buildRuntimeModule() {
   // Ensure that the declarations exist before creating llvm types for them.
   ensureDecl(ClassDeclaration::object, "Object");
   ensureDecl(Type::typeinfoclass, "TypeInfo_Class");
-  ensureDecl(Type::dtypeinfo, "DTypeInfo");
+  ensureDecl(Type::dtypeinfo, "TypeInfo");
   ensureDecl(Type::typeinfoassociativearray, "TypeInfo_AssociativeArray");
   ensureDecl(Module::moduleinfo, "ModuleInfo");
 
@@ -306,8 +306,9 @@ static void buildRuntimeModule() {
   //////////////////////////////////////////////////////////////////////////////
 
   // Construct some attribute lists used below (possibly multiple times)
-  AttrSet NoAttrs, Attr_NoAlias(NoAttrs, LLAttributeSet::ReturnIndex,
-                                llvm::Attribute::NoAlias),
+  AttrSet NoAttrs,
+      Attr_NoAlias(NoAttrs, LLAttributeSet::ReturnIndex,
+                   llvm::Attribute::NoAlias),
       Attr_NoUnwind(NoAttrs, LLAttributeSet::FunctionIndex,
                     llvm::Attribute::NoUnwind),
       Attr_ReadOnly(NoAttrs, LLAttributeSet::FunctionIndex,
@@ -317,19 +318,29 @@ static void buildRuntimeModule() {
                          llvm::Attribute::NoReturn),
       Attr_ReadOnly_NoUnwind(Attr_ReadOnly, LLAttributeSet::FunctionIndex,
                              llvm::Attribute::NoUnwind),
-      Attr_ReadOnly_1_NoCapture(Attr_ReadOnly, 1, llvm::Attribute::NoCapture),
-      Attr_ReadOnly_1_3_NoCapture(Attr_ReadOnly_1_NoCapture, 3,
+      Attr_ReadOnly_1_NoCapture(Attr_ReadOnly, AttrSet::FirstArgIndex,
+                                llvm::Attribute::NoCapture),
+      Attr_ReadOnly_1_3_NoCapture(Attr_ReadOnly_1_NoCapture,
+                                  AttrSet::FirstArgIndex + 2,
                                   llvm::Attribute::NoCapture),
-      Attr_ReadOnly_NoUnwind_1_NoCapture(Attr_ReadOnly_1_NoCapture, ~0U,
+      Attr_ReadOnly_NoUnwind_1_NoCapture(Attr_ReadOnly_1_NoCapture,
+                                         LLAttributeSet::FunctionIndex,
                                          llvm::Attribute::NoUnwind),
       Attr_ReadOnly_NoUnwind_1_2_NoCapture(Attr_ReadOnly_NoUnwind_1_NoCapture,
-                                           2, llvm::Attribute::NoCapture),
-      Attr_ReadNone(NoAttrs, ~0U, llvm::Attribute::ReadNone),
-      Attr_1_NoCapture(NoAttrs, 1, llvm::Attribute::NoCapture),
-      Attr_NoAlias_1_NoCapture(Attr_1_NoCapture, 0, llvm::Attribute::NoAlias),
-      Attr_1_2_NoCapture(Attr_1_NoCapture, 2, llvm::Attribute::NoCapture),
-      Attr_1_3_NoCapture(Attr_1_NoCapture, 3, llvm::Attribute::NoCapture),
-      Attr_1_4_NoCapture(Attr_1_NoCapture, 4, llvm::Attribute::NoCapture);
+                                           AttrSet::FirstArgIndex + 1,
+                                           llvm::Attribute::NoCapture),
+      Attr_ReadNone(NoAttrs, LLAttributeSet::FunctionIndex,
+                    llvm::Attribute::ReadNone),
+      Attr_1_NoCapture(NoAttrs, AttrSet::FirstArgIndex,
+                       llvm::Attribute::NoCapture),
+      Attr_1_2_NoCapture(Attr_1_NoCapture, AttrSet::FirstArgIndex + 1,
+                         llvm::Attribute::NoCapture),
+      Attr_1_3_NoCapture(Attr_1_NoCapture, AttrSet::FirstArgIndex + 2,
+                         llvm::Attribute::NoCapture),
+      Attr_1_4_NoCapture(Attr_1_NoCapture, AttrSet::FirstArgIndex + 3,
+                         llvm::Attribute::NoCapture),
+      Attr_NoAlias_1_NoCapture(Attr_1_NoCapture, LLAttributeSet::ReturnIndex,
+                               llvm::Attribute::NoAlias);
 
   //////////////////////////////////////////////////////////////////////////////
   //////////////////////////////////////////////////////////////////////////////
@@ -702,7 +713,8 @@ static void buildRuntimeModule() {
                 {objectTy});
 
   // void _d_dso_registry(void* data)
-  // (the argument is really a pointer to rt.sections_elf_shared.CompilerDSOData)
+  // (the argument is really a pointer to
+  // rt.sections_elf_shared.CompilerDSOData)
   createFwdDecl(LINKc, voidTy, {"_d_dso_registry"}, {voidPtrTy});
 
   //////////////////////////////////////////////////////////////////////////////
@@ -772,11 +784,7 @@ static void emitInstrumentationFn(const char *name) {
       gIR->ir->CreateCall(GET_INTRINSIC_DECL(returnaddress), DtoConstInt(0));
   auto callee = DtoBitCast(gIR->topfunc(), getVoidPtrType());
 
-#if LDC_LLVM_VER >= 307
   gIR->ir->CreateCall(fn, {callee, caller});
-#else
-  gIR->ir->CreateCall2(fn, callee, caller);
-#endif
 }
 
 void emitInstrumentationFnEnter(FuncDeclaration *decl) {
