@@ -27,6 +27,7 @@
 #include "gen/runtime.h"
 #include "ir/irfunction.h"
 #include "ir/irtype.h"
+#include "llvm/IR/LLVMContext.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -197,8 +198,7 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     }
 
     args[llArgIdx] = llVal;
-    // +1 as index 0 contains the function attributes.
-    attrs.add(llArgIdx + 1, irArg->attrs);
+    attrs.addToParam(llArgIdx, irArg->attrs);
 
     if (isVararg) {
       delete irArg;
@@ -333,11 +333,18 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     }
     auto atomicOrdering =
         static_cast<llvm::AtomicOrdering>((*e->arguments)[0]->toInteger());
+#if LDC_LLVM_VER >= 500
+    llvm::SyncScope::ID scope = llvm::SyncScope::System;
+    if (e->arguments->dim == 2) {
+      scope = static_cast<llvm::SyncScope::ID>((*e->arguments)[1]->toInteger());
+    }
+#else
     auto scope = llvm::SynchronizationScope::CrossThread;
     if (e->arguments->dim == 2) {
       scope = static_cast<llvm::SynchronizationScope>(
           (*e->arguments)[1]->toInteger());
     }
+#endif
     p->ir->CreateFence(atomicOrdering, scope);
     return true;
   }
@@ -688,10 +695,11 @@ private:
     }
 
     args.push_back(pointer);
-    attrs.add(index + 1, irFty.arg_sret->attrs);
+    attrs.addToParam(index, irFty.arg_sret->attrs);
 
     // verify that sret and/or inreg attributes are set
     const AttrBuilder &sretAttrs = irFty.arg_sret->attrs;
+    (void)sretAttrs;
     assert((sretAttrs.contains(LLAttribute::StructRet) ||
             sretAttrs.contains(LLAttribute::InReg)) &&
            "Sret arg not sret or inreg?");
@@ -759,9 +767,9 @@ private:
 
     // add attributes
     if (irFty.arg_this) {
-      attrs.add(index + 1, irFty.arg_this->attrs);
+      attrs.addToParam(index, irFty.arg_this->attrs);
     } else if (irFty.arg_nest) {
-      attrs.add(index + 1, irFty.arg_nest->attrs);
+      attrs.addToParam(index, irFty.arg_nest->attrs);
     }
 
     if (irFty.arg_objcSelector && dfnval) {
@@ -784,7 +792,7 @@ private:
         getTypeinfoArrayArgumentForDVarArg(argvals, numFormalParams);
 
     args.push_back(argumentsArg);
-    attrs.add(args.size(), irFty.arg_arguments->attrs);
+    attrs.addToParam(args.size() - 1, irFty.arg_arguments->attrs);
   }
 };
 
@@ -826,7 +834,7 @@ DValue *DtoCallFunctionImpl(Loc &loc, Type *resulttype, DValue *fnval,
   AttrSet attrs;
 
   // return attrs
-  attrs.add(0, irFty.ret->attrs);
+  attrs.addToReturn(irFty.ret->attrs);
 
   std::vector<LLValue *> args;
   args.reserve(irFty.args.size());
@@ -994,9 +1002,14 @@ DValue *DtoCallFunctionImpl(Loc &loc, Type *resulttype, DValue *fnval,
     call.setCallingConv(callconv);
   }
   // merge in function attributes set in callOrInvoke
+#if LDC_LLVM_VER >= 500
+  attrlist = attrlist.addAttributes(
+      gIR->context(), LLAttributeSet::FunctionIndex,
+      llvm::AttrBuilder(call.getAttributes(), LLAttributeSet::FunctionIndex));
+#else
   attrlist = attrlist.addAttributes(
       gIR->context(), LLAttributeSet::FunctionIndex, call.getAttributes());
-
+#endif
   call.setAttributes(attrlist);
 
   // Special case for struct constructor calls: For temporaries, using the
