@@ -157,8 +157,8 @@ public:
                          e->type->toChars());
     LOG_SCOPE;
 
-    Type * const t = e->type->toBasetype();
-    Type * const cty = t->nextOf()->toBasetype();
+    Type *const t = e->type->toBasetype();
+    Type *const cty = t->nextOf()->toBasetype();
 
     auto _init = buildStringLiteralConstant(e, t->ty != Tsarray);
 
@@ -189,17 +189,14 @@ public:
     llvm::ConstantInt *zero =
         LLConstantInt::get(LLType::getInt32Ty(gIR->context()), 0, false);
     LLConstant *idxs[2] = {zero, zero};
-#if LDC_LLVM_VER >= 307
     LLConstant *arrptr = llvm::ConstantExpr::getGetElementPtr(
         isaPointer(gvar)->getElementType(), gvar, idxs, true);
-#else
-    LLConstant *arrptr = llvm::ConstantExpr::getGetElementPtr(gvar, idxs, true);
-#endif
 
     if (t->ty == Tpointer) {
       result = arrptr;
     } else if (t->ty == Tarray) {
-      LLConstant *clen = LLConstantInt::get(DtoSize_t(), e->numberOfCodeUnits(), false);
+      LLConstant *clen =
+          LLConstantInt::get(DtoSize_t(), e->numberOfCodeUnits(), false);
       result = DtoConstSlice(clen, arrptr, e->type);
     } else {
       llvm_unreachable("Unknown type for StringExp.");
@@ -218,12 +215,8 @@ public:
     if (t1b->ty == Tpointer && e->e2->type->isintegral()) {
       llvm::Constant *ptr = toConstElem(e->e1);
       dinteger_t idx = undoStrideMul(e->loc, t1b, e->e2->toInteger());
-#if LDC_LLVM_VER >= 307
       result = llvm::ConstantExpr::getGetElementPtr(
           isaPointer(ptr)->getElementType(), ptr, DtoConstSize_t(idx));
-#else
-      result = llvm::ConstantExpr::getGetElementPtr(ptr, DtoConstSize_t(idx));
-#endif
     } else {
       e->error("expression '%s' is not a constant", e->toChars());
       if (!global.gag) {
@@ -244,12 +237,8 @@ public:
       dinteger_t idx = undoStrideMul(e->loc, t1b, e->e2->toInteger());
 
       llvm::Constant *negIdx = llvm::ConstantExpr::getNeg(DtoConstSize_t(idx));
-#if LDC_LLVM_VER >= 307
       result = llvm::ConstantExpr::getGetElementPtr(
           isaPointer(ptr)->getElementType(), ptr, negIdx);
-#else
-      result = llvm::ConstantExpr::getGetElementPtr(ptr, negIdx);
-#endif
     } else {
       e->error("expression '%s' is not a constant", e->toChars());
       if (!global.gag) {
@@ -303,12 +292,8 @@ public:
       Type *type = vd->type->toBasetype();
       if (type->ty == Tarray || type->ty == Tdelegate) {
         LLConstant *idxs[2] = {DtoConstSize_t(0), DtoConstSize_t(1)};
-#if LDC_LLVM_VER >= 307
         value = llvm::ConstantExpr::getGetElementPtr(
             isaPointer(value)->getElementType(), value, idxs, true);
-#else
-        value = llvm::ConstantExpr::getGetElementPtr(value, idxs, true);
-#endif
       }
       result = DtoBitCast(value, DtoType(tb));
     } else if (tb->ty == Tclass && e->e1->type->ty == Tclass &&
@@ -368,18 +353,12 @@ public:
 
       if (e->offset % elemSize == 0) {
         // We can turn this into a "nice" GEP.
-        result = llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
-            nullptr,
-#endif
+        result = llvm::ConstantExpr::getGetElementPtr(nullptr,
             base, DtoConstSize_t(e->offset / elemSize));
       } else {
         // Offset isn't a multiple of base type size, just cast to i8* and
         // apply the byte offset.
-        result = llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
-            nullptr,
-#endif
+        result = llvm::ConstantExpr::getGetElementPtr(nullptr,
             DtoBitCast(base, getVoidPtrType()), DtoConstSize_t(e->offset));
       }
     }
@@ -423,12 +402,8 @@ public:
       LLConstant *idxs[2] = {DtoConstSize_t(0), index};
       LLConstant *val = isaConstant(getIrGlobal(vd)->value);
       val = DtoBitCast(val, DtoType(vd->type->pointerTo()));
-#if LDC_LLVM_VER >= 307
       LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(
           isaPointer(val)->getElementType(), val, idxs, true);
-#else
-      LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(val, idxs, true);
-#endif
 
       // bitcast to requested type
       assert(e->type->toBasetype()->ty == Tpointer);
@@ -437,29 +412,20 @@ public:
       StructLiteralExp *se = static_cast<StructLiteralExp *>(e->e1);
 
       if (se->globalVar) {
-        IF_LOG Logger::cout() << "Returning existing global: " << *se->globalVar
-                              << '\n';
+        IF_LOG Logger::cout()
+            << "Returning existing global: " << *se->globalVar << '\n';
         result = se->globalVar;
         return;
       }
 
-      se->globalVar = new llvm::GlobalVariable(
-          p->module, DtoType(e->e1->type), false,
+      auto globalVar = new llvm::GlobalVariable(
+          p->module, DtoType(se->type), false,
           llvm::GlobalValue::InternalLinkage, nullptr, ".structliteral");
+      globalVar->setAlignment(DtoAlignment(se->type));
 
+      se->globalVar = globalVar;
       llvm::Constant *constValue = toConstElem(se);
-      if (constValue->getType() !=
-          se->globalVar->getType()->getContainedType(0)) {
-        auto finalGlobalVar = new llvm::GlobalVariable(
-            p->module, constValue->getType(), false,
-            llvm::GlobalValue::InternalLinkage, nullptr, ".structliteral");
-        se->globalVar->replaceAllUsesWith(
-            DtoBitCast(finalGlobalVar, se->globalVar->getType()));
-        se->globalVar->eraseFromParent();
-        se->globalVar = finalGlobalVar;
-      }
-      se->globalVar->setInitializer(constValue);
-      se->globalVar->setAlignment(DtoAlignment(se->type));
+      se->globalVar = p->setGlobalVarInitializer(globalVar, constValue);
 
       result = se->globalVar;
     } else if (e->e1->op == TOKslice) {
@@ -545,7 +511,8 @@ public:
         gIR->module, initval->getType(), canBeConst,
         llvm::GlobalValue::InternalLinkage, initval, ".dynarrayStorage");
 #if LDC_LLVM_VER >= 309
-    gvar->setUnnamedAddr(canBeConst ? llvm::GlobalValue::UnnamedAddr::Global : llvm::GlobalValue::UnnamedAddr::None);
+    gvar->setUnnamedAddr(canBeConst ? llvm::GlobalValue::UnnamedAddr::Global
+                                    : llvm::GlobalValue::UnnamedAddr::None);
 #else
     gvar->setUnnamedAddr(canBeConst);
 #endif
@@ -560,13 +527,8 @@ public:
     // build a constant dynamic array reference with the .ptr field pointing
     // into store
     LLConstant *idxs[2] = {DtoConstUint(0), DtoConstUint(0)};
-#if LDC_LLVM_VER >= 307
     LLConstant *globalstorePtr = llvm::ConstantExpr::getGetElementPtr(
         isaPointer(store)->getElementType(), store, idxs, true);
-#else
-    LLConstant *globalstorePtr =
-        llvm::ConstantExpr::getGetElementPtr(store, idxs, true);
-#endif
 
     result = DtoConstSlice(DtoConstSize_t(e->elements->dim), globalstorePtr);
   }
@@ -589,8 +551,8 @@ public:
       std::map<VarDeclaration *, llvm::Constant *> varInits;
       const size_t nexprs = e->elements->dim;
       for (size_t i = 0; i < nexprs; i++) {
-        if ((*e->elements)[i]) {
-          LLConstant *c = toConstElem((*e->elements)[i]);
+        if (auto elem = (*e->elements)[i]) {
+          LLConstant *c = toConstElem(elem);
           // extend i1 to i8
           if (c->getType() == LLType::getInt1Ty(p->context()))
             c = llvm::ConstantExpr::getZExt(c, LLType::getInt8Ty(p->context()));
@@ -614,12 +576,13 @@ public:
     StructLiteralExp *value = e->value;
 
     if (value->globalVar) {
-      IF_LOG Logger::cout() << "Using existing global: " << *value->globalVar
-                            << '\n';
+      IF_LOG Logger::cout()
+          << "Using existing global: " << *value->globalVar << '\n';
     } else {
-      value->globalVar = new llvm::GlobalVariable(
+      auto globalVar = new llvm::GlobalVariable(
           p->module, origClass->type->ctype->isClass()->getMemoryLLType(),
           false, llvm::GlobalValue::InternalLinkage, nullptr, ".classref");
+      value->globalVar = globalVar;
 
       std::map<VarDeclaration *, llvm::Constant *> varInits;
 
@@ -639,33 +602,25 @@ public:
           cur = classHierachy.top();
           classHierachy.pop();
           for (size_t j = 0; j < cur->fields.dim; ++j) {
-            if ((*value->elements)[i]) {
+            if (auto elem = (*value->elements)[i]) {
               VarDeclaration *field = cur->fields[j];
               IF_LOG Logger::println("Getting initializer for: %s",
                                      field->toChars());
               LOG_SCOPE;
-              varInits[field] = toConstElem((*value->elements)[i]);
+              varInits[field] = toConstElem(elem);
             }
             ++i;
           }
         }
+
+        (void)nexprs;
         assert(i == nexprs);
       }
 
       llvm::Constant *constValue =
           getIrAggr(origClass)->createInitializerConstant(varInits);
 
-      if (constValue->getType() !=
-          value->globalVar->getType()->getContainedType(0)) {
-        auto finalGlobalVar = new llvm::GlobalVariable(
-            p->module, constValue->getType(), false,
-            llvm::GlobalValue::InternalLinkage, nullptr, ".classref");
-        value->globalVar->replaceAllUsesWith(
-            DtoBitCast(finalGlobalVar, value->globalVar->getType()));
-        value->globalVar->eraseFromParent();
-        value->globalVar = finalGlobalVar;
-      }
-      value->globalVar->setInitializer(constValue);
+      value->globalVar = p->setGlobalVarInitializer(globalVar, constValue);
     }
 
     result = value->globalVar;
@@ -700,20 +655,35 @@ public:
     TypeVector *tv = static_cast<TypeVector *>(e->to->toBasetype());
     assert(tv->ty == Tvector);
 
-    // The AST for
-    //   static immutable ubyte16 vec1 = 123;
-    // differs from
-    //    static immutable ubyte[16] vec1 = 123;
-    // In the vector case the AST contains an IntegerExp (of type int) and a
-    // CastExp to type ubyte. In the static array case the AST only contains an
-    // IntegerExp of type ubyte. Simply call optimize to get  rid of the cast.
-    // FIXME: Check DMD source to understand why two different ASTs are
-    //        constructed.
-    llvm::Constant *val = toConstElem(e->e1->optimize(WANTvalue));
-
-    dinteger_t elemCount =
+    const auto elemCount =
         static_cast<TypeSArray *>(tv->basetype)->dim->toInteger();
-    result = llvm::ConstantVector::getSplat(elemCount, val);
+
+    // Array literals are assigned element-for-element; other expressions splat
+    // across the whole vector.
+    if (e->e1->op == TOKarrayliteral) {
+      const auto ale = static_cast<ArrayLiteralExp *>(e->e1);
+
+      llvm::SmallVector<llvm::Constant *, 16> elements;
+      elements.reserve(elemCount);
+      for (size_t i = 0; i < elemCount; ++i) {
+        elements.push_back(toConstElem(indexArrayLiteral(ale, i)));
+      }
+
+      result = llvm::ConstantVector::get(elements);
+    } else {
+      // The AST for
+      //   static immutable ubyte16 vec1 = 123;
+      // differs from
+      //    static immutable ubyte[16] vec1 = 123;
+      // In the vector case the AST contains an IntegerExp (of type int) and a
+      // CastExp to type ubyte. In the static array case the AST only contains
+      // an IntegerExp of type ubyte. Simply call optimize to get rid of the
+      // cast.
+      // FIXME: Check DMD source to understand why two different ASTs are
+      //        constructed.
+      result = llvm::ConstantVector::getSplat(
+          elemCount, toConstElem(e->e1->optimize(WANTvalue)));
+    }
   }
 
   //////////////////////////////////////////////////////////////////////////////
