@@ -56,9 +56,8 @@
 #include "ctfe.h"
 
 llvm::cl::opt<bool> checkPrintf(
-    "check-printf-calls",
-    llvm::cl::desc("Validate printf call format strings against arguments"),
-    llvm::cl::ZeroOrMore);
+    "check-printf-calls", llvm::cl::ZeroOrMore,
+    llvm::cl::desc("Validate printf call format strings against arguments"));
 
 bool walkPostorder(Expression *e, StoppableVisitor *v);
 
@@ -77,6 +76,7 @@ static void write_struct_literal(Loc loc, LLValue *mem, StructDeclaration *sd,
                                  Expressions *elements) {
   assert(elements && "struct literal has null elements");
   const auto numMissingElements = sd->fields.dim - elements->dim;
+  (void)numMissingElements;
   assert(numMissingElements == 0 || (sd->vthis && numMissingElements == 1));
 
   // might be reset to an actual i8* value so only a single bitcast is emitted
@@ -414,12 +414,8 @@ public:
     llvm::ConstantInt *zero =
         LLConstantInt::get(LLType::getInt32Ty(gIR->context()), 0, false);
     LLConstant *idxs[2] = {zero, zero};
-#if LDC_LLVM_VER >= 307
     LLConstant *arrptr = llvm::ConstantExpr::getGetElementPtr(
         isaPointer(gvar)->getElementType(), gvar, idxs, true);
-#else
-    LLConstant *arrptr = llvm::ConstantExpr::getGetElementPtr(gvar, idxs, true);
-#endif
 
     if (dtype->ty == Tarray) {
       LLConstant *clen =
@@ -1072,19 +1068,23 @@ public:
     PGO.setCurrentStmt(e);
 
     // value being sliced
+    Type *const etype = e->e1->type->toBasetype();
+    LLValue *eptr = nullptr;
     LLValue *elen = nullptr;
-    LLValue *eptr;
-    DValue *v = toElem(e->e1);
 
-    Type *etype = e->e1->type->toBasetype();
-    if (etype->ty == Tpointer) {
-      // pointer slicing
-      assert(e->lwr);
-      eptr = DtoRVal(v);
-    } else {
-      // array slice
-      eptr = DtoArrayPtr(v);
-    }
+    // evaluate the base expression but delay getting its pointer until the
+    // potential bounds have been evaluated
+    DValue *v = toElem(e->e1);
+    auto getBasePointer = [e, v, etype]() {
+      if (etype->ty == Tpointer) {
+        // pointer slicing
+        assert(e->lwr);
+        return DtoRVal(v);
+      } else {
+        // array slice
+        return DtoArrayPtr(v);
+      }
+    };
 
     // has lower bound, pointer needs adjustment
     if (e->lwr) {
@@ -1133,14 +1133,15 @@ public:
       }
 
       // offset by lower
-      eptr = DtoGEP1(eptr, vlo, !needCheckLower, "lowerbound");
+      eptr = DtoGEP1(getBasePointer(), vlo, !needCheckLower, "lowerbound");
 
       // adjust length
       elen = p->ir->CreateSub(vup, vlo);
     }
     // no bounds or full slice -> just convert to slice
     else {
-      assert(e->e1->type->toBasetype()->ty != Tpointer);
+      assert(etype->ty != Tpointer);
+      eptr = getBasePointer();
       // if the slicee is a static array, we use the length of that as DMD seems
       // to give contrary inconsistent sizesin some multidimensional static
       // array cases.
@@ -1466,8 +1467,8 @@ public:
         size_t ndims = e->arguments->dim;
         std::vector<DValue *> dims;
         dims.reserve(ndims);
-        for (size_t i = 0; i < ndims; ++i) {
-          dims.push_back(toElem((*e->arguments)[i]));
+        for (auto arg : *e->arguments) {
+          dims.push_back(toElem(arg));
         }
         result = DtoNewMulDimDynArray(e->loc, e->newtype, &dims[0], ndims);
       }
@@ -1551,6 +1552,7 @@ public:
       result = new DImValue(e->type, mem);
     }
 
+    (void)isArgprefixHandled;
     assert(e->argprefix == NULL || isArgprefixHandled);
   }
 
@@ -1643,17 +1645,8 @@ public:
     DValue *cond;
     Type *condty;
 
-    // special case for dmd generated assert(this); when not in -release mode
-    if (e->e1->op == TOKthis && static_cast<ThisExp *>(e->e1)->var == nullptr) {
-      LLValue *thisarg = p->func()->thisArg;
-      assert(thisarg && "null thisarg, but we're in assert(this) exp;");
-      LLValue *thisptr = DtoLoad(thisarg);
-      condty = e->e1->type->toBasetype();
-      cond = new DImValue(condty, thisptr);
-    } else {
-      cond = toElem(e->e1);
-      condty = e->e1->type->toBasetype();
-    }
+    cond = toElem(e->e1);
+    condty = e->e1->type->toBasetype();
 
     // create basic blocks
     llvm::BasicBlock *passedbb = p->insertBB("assertPassed");
@@ -1681,30 +1674,35 @@ public:
     // passed:
     p->scope() = IRScope(passedbb);
 
-    FuncDeclaration *invdecl;
-    // class invariants
-    if (global.params.useInvariants && condty->ty == Tclass &&
-        !(static_cast<TypeClass *>(condty)->sym->isInterfaceDeclaration()) &&
-        !(static_cast<TypeClass *>(condty)->sym->isCPPclass())) {
+    // class/struct invariants
+    if (!global.params.useInvariants)
+      return;
+    if (condty->ty == Tclass) {
+      const auto sym = static_cast<TypeClass *>(condty)->sym;
+      if (sym->isInterfaceDeclaration() || sym->isCPPclass())
+        return;
+
       Logger::println("calling class invariant");
-      llvm::Function *fn = getRuntimeFunction(
-          e->loc, gIR->module,
-          gABI->mangleFunctionForLLVM("_D9invariant12_d_invariantFC6ObjectZv",
-                                      LINKd)
-              .c_str());
-      LLValue *arg =
+
+      const auto fnMangle = gABI->mangleFunctionForLLVM(
+          "_D9invariant12_d_invariantFC6ObjectZv", LINKd);
+      const auto fn = getRuntimeFunction(e->loc, gIR->module, fnMangle.c_str());
+
+      const auto arg =
           DtoBitCast(DtoRVal(cond), fn->getFunctionType()->getParamType(0));
+
       gIR->CreateCallOrInvoke(fn, arg);
-    }
-    // struct invariants
-    else if (global.params.useInvariants && condty->ty == Tpointer &&
-             condty->nextOf()->ty == Tstruct &&
-             (invdecl = static_cast<TypeStruct *>(condty->nextOf())
-                            ->sym->inv) != nullptr) {
+    } else if (condty->ty == Tpointer && condty->nextOf()->ty == Tstruct) {
+      const auto invDecl =
+          static_cast<TypeStruct *>(condty->nextOf())->sym->inv;
+      if (!invDecl)
+        return;
+
       Logger::print("calling struct invariant");
-      DtoResolveFunction(invdecl);
-      DFuncValue invfunc(invdecl, DtoCallee(invdecl), DtoRVal(cond));
-      DtoCallFunction(e->loc, nullptr, &invfunc, nullptr);
+
+      DtoResolveFunction(invDecl);
+      DFuncValue invFunc(invDecl, DtoCallee(invDecl), DtoRVal(cond));
+      DtoCallFunction(e->loc, nullptr, &invFunc, nullptr);
     }
   }
 
@@ -1854,11 +1852,7 @@ public:
     IF_LOG Logger::print("HaltExp::toElem: %s\n", e->toChars());
     LOG_SCOPE;
 
-#if LDC_LLVM_VER >= 307
     p->ir->CreateCall(GET_INTRINSIC_DECL(trap), {});
-#else
-    p->ir->CreateCall(GET_INTRINSIC_DECL(trap), "");
-#endif
     p->ir->CreateUnreachable();
 
     // this terminated the basicblock, start a new one
@@ -2218,8 +2212,7 @@ public:
       }
       cval = DtoBitCast(cval, dgty->getContainedType(0));
 
-      LLValue *castfptr =
-          DtoBitCast(DtoCallee(fd), dgty->getContainedType(1));
+      LLValue *castfptr = DtoBitCast(DtoCallee(fd), dgty->getContainedType(1));
 
       result = new DImValue(e->type, DtoAggrPair(cval, castfptr, ".func"));
 
@@ -2267,7 +2260,7 @@ public:
         auto global = new llvm::GlobalVariable(
             gIR->module, init->getType(), true,
             llvm::GlobalValue::InternalLinkage, init, ".immutablearray");
-        result = new DSliceValue(arrayType, DtoConstSize_t(e->elements->dim),
+        result = new DSliceValue(arrayType, DtoConstSize_t(len),
                                  DtoBitCast(global, getPtrToType(llElemType)));
       } else {
         DSliceValue *dynSlice = DtoNewDynArray(
@@ -2426,8 +2419,8 @@ public:
       keysInits.reserve(e->keys->dim);
       valuesInits.reserve(e->keys->dim);
       for (size_t i = 0, n = e->keys->dim; i < n; ++i) {
-        Expression *ekey = e->keys->tdata()[i];
-        Expression *eval = e->values->tdata()[i];
+        Expression *ekey = (*e->keys)[i];
+        Expression *eval = (*e->values)[i];
         IF_LOG Logger::println("(%llu) aa[%s] = %s",
                                static_cast<unsigned long long>(i),
                                ekey->toChars(), eval->toChars());
@@ -2459,13 +2452,8 @@ public:
       LLConstant *globalstore = new LLGlobalVariable(
           gIR->module, initval->getType(), false,
           LLGlobalValue::InternalLinkage, initval, ".aaKeysStorage");
-#if LDC_LLVM_VER >= 307
       LLConstant *slice = llvm::ConstantExpr::getGetElementPtr(
           isaPointer(globalstore)->getElementType(), globalstore, idxs, true);
-#else
-      LLConstant *slice =
-          llvm::ConstantExpr::getGetElementPtr(globalstore, idxs, true);
-#endif
       slice = DtoConstSlice(DtoConstSize_t(e->keys->dim), slice);
       LLValue *keysArray = DtoAggrPaint(slice, funcTy->getParamType(1));
 
@@ -2473,12 +2461,8 @@ public:
       globalstore = new LLGlobalVariable(gIR->module, initval->getType(), false,
                                          LLGlobalValue::InternalLinkage,
                                          initval, ".aaValuesStorage");
-#if LDC_LLVM_VER >= 307
       slice = llvm::ConstantExpr::getGetElementPtr(
           isaPointer(globalstore)->getElementType(), globalstore, idxs, true);
-#else
-      slice = llvm::ConstantExpr::getGetElementPtr(globalstore, idxs, true);
-#endif
       slice = DtoConstSlice(DtoConstSize_t(e->keys->dim), slice);
       LLValue *valuesArray = DtoAggrPaint(slice, funcTy->getParamType(2));
 
@@ -2580,8 +2564,8 @@ public:
 
     std::vector<LLType *> types;
     types.reserve(e->exps->dim);
-    for (size_t i = 0; i < e->exps->dim; i++) {
-      types.push_back(DtoMemType((*e->exps)[i]->type));
+    for (auto exp : *e->exps) {
+      types.push_back(DtoMemType(exp->type));
     }
     LLValue *val =
         DtoRawAlloca(LLStructType::get(gIR->context(), types), 0, ".tuple");
