@@ -56,24 +56,9 @@ bool DtoIsReturnInArg(CallExp *ce) {
 
 LLAttribute DtoShouldExtend(Type *type) {
   type = type->toBasetype();
-  if (type->isintegral()) {
-    switch (type->ty) {
-    case Tint8:
-    case Tint16:
-      return LLAttribute::SExt;
-
-    case Tuns8:
-    case Tuns16:
-    case Tchar:
-    case Twchar:
-      return LLAttribute::ZExt;
-
-    default:
-      // Do not extend.
-      break;
-    }
+  if (type->isintegral() && type->ty != Tvector && type->size() <= 2) {
+    return type->isunsigned() ? LLAttribute::ZExt : LLAttribute::SExt;
   }
-
   return LLAttribute::None;
 }
 
@@ -249,11 +234,7 @@ LinkageWithCOMDAT DtoLinkage(Dsymbol *sym) {
 }
 
 bool supportsCOMDAT() {
-#if LDC_LLVM_VER >= 307
   return !global.params.targetTriple->isOSBinFormatMachO();
-#else
-  return false;
-#endif
 }
 
 void setLinkage(LinkageWithCOMDAT lwc, llvm::GlobalObject *obj) {
@@ -272,8 +253,17 @@ LLIntegerType *DtoSize_t() {
   // the type of size_t does not change once set
   static LLIntegerType *t = nullptr;
   if (t == nullptr) {
-    t = (global.params.isLP64) ? LLType::getInt64Ty(gIR->context())
-                               : LLType::getInt32Ty(gIR->context());
+    auto triple = global.params.targetTriple;
+
+    if (triple->isArch64Bit()) {
+      t = LLType::getInt64Ty(gIR->context());
+    } else if (triple->isArch32Bit()) {
+      t = LLType::getInt32Ty(gIR->context());
+    } else if (triple->isArch16Bit()) {
+      t = LLType::getInt16Ty(gIR->context());
+    } else {
+      llvm_unreachable("Unsupported size_t width");
+    }
   }
   return t;
 }
@@ -285,12 +275,10 @@ llvm::GetElementPtrInst *DtoGEP(LLValue *ptr, llvm::ArrayRef<LLValue *> indices,
                                 bool inBounds, const char *name,
                                 llvm::BasicBlock *bb) {
   LLPointerType *p = isaPointer(ptr);
+  (void)p;
   assert(p && "GEP expects a pointer type");
   auto gep = llvm::GetElementPtrInst::Create(
-#if LDC_LLVM_VER >= 307
-      p->getElementType(),
-#endif
-      ptr, indices, name, bb ? bb : gIR->scopebb());
+      p->getElementType(), ptr, indices, name, bb ? bb : gIR->scopebb());
   gep->setIsInBounds(inBounds);
   return gep;
 }
@@ -320,13 +308,11 @@ LLValue *DtoGEPi(LLValue *ptr, unsigned i0, unsigned i1, const char *name,
 
 LLConstant *DtoGEPi(LLConstant *ptr, unsigned i0, unsigned i1) {
   LLPointerType *p = isaPointer(ptr);
+  (void)p;
   assert(p && "GEP expects a pointer type");
   LLValue *indices[] = {DtoConstUint(i0), DtoConstUint(i1)};
   return llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
-      p->getElementType(),
-#endif
-      ptr, indices, /* InBounds = */ true);
+      p->getElementType(), ptr, indices, /* InBounds = */ true);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -386,11 +372,7 @@ LLValue *DtoMemCmp(LLValue *lhs, LLValue *rhs, LLValue *nbytes) {
   lhs = DtoBitCast(lhs, VoidPtrTy);
   rhs = DtoBitCast(rhs, VoidPtrTy);
 
-#if LDC_LLVM_VER >= 307
   return gIR->ir->CreateCall(fn, {lhs, rhs, nbytes});
-#else
-  return gIR->ir->CreateCall3(fn, lhs, rhs, nbytes);
-#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -415,31 +397,15 @@ LLConstant *DtoConstFP(Type *t, const real_t value) {
   LLType *llty = DtoType(t);
   assert(llty->isFloatingPointTy());
 
-  assert(sizeof(real_t) >= 8 && "real_t < 64 bits?");
-
-  if (llty->isFloatTy()) {
-    // let host narrow to single-precision target
-    return LLConstantFP::get(gIR->context(),
-                             APFloat(static_cast<float>(value)));
-  }
-
-  if (llty->isDoubleTy()) {
-    // let host (potentially) narrow to double-precision target
-    return LLConstantFP::get(gIR->context(),
-                             APFloat(static_cast<double>(value)));
-  }
-
-  // host real_t => target real
-
   // 1) represent host real_t as llvm::APFloat
-  const auto &targetRealSemantics = llty->getFltSemantics();
-  APFloat v(targetRealSemantics, APFloat::uninitialized);
+  const auto &targetSemantics = llty->getFltSemantics();
+  APFloat v(targetSemantics, APFloat::uninitialized);
   CTFloat::toAPFloat(value, v);
 
-  // 2) convert to target real
-  if (&v.getSemantics() != &targetRealSemantics) {
+  // 2) convert to target format
+  if (&v.getSemantics() != &targetSemantics) {
     bool ignored;
-    v.convert(targetRealSemantics, APFloat::rmNearestTiesToEven, &ignored);
+    v.convert(targetSemantics, APFloat::rmNearestTiesToEven, &ignored);
   }
 
   return LLConstantFP::get(gIR->context(), v);
@@ -469,9 +435,7 @@ LLConstant *DtoConstString(const char *str) {
   LLConstant *idxs[] = {DtoConstUint(0), DtoConstUint(0)};
   return DtoConstSlice(DtoConstSize_t(s.size()),
                        llvm::ConstantExpr::getGetElementPtr(
-#if LDC_LLVM_VER >= 307
                            gvar->getInitializer()->getType(),
-#endif
                            gvar, idxs, true),
                        Type::tchar->arrayOf());
 }
@@ -528,8 +492,31 @@ void DtoAlignedStore(LLValue *src, LLValue *dst) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+LLType *stripAddrSpaces(LLType *t)
+{
+  // Fastpath for normal compilation.
+  if(gIR->dcomputetarget == nullptr)
+    return t;
+
+  int indirections = 0;
+  while (t->isPointerTy()) {
+    indirections++;
+    t = t->getPointerElementType();
+  }
+  while (indirections-- != 0)
+     t = t->getPointerTo(0);
+
+  return t;
+}
+
 LLValue *DtoBitCast(LLValue *v, LLType *t, const llvm::Twine &name) {
-  if (v->getType() == t) {
+  // Strip addrspace qualifications from v before comparing types by pointer
+  // equality. This avoids the case where the pointer in { T addrspace(n)* }
+  // is dereferenced and generates a GEP -> (invalid) bitcast -> load sequence.
+  // Bitcasting of pointers between addrspaces is invalid in LLVM IR. Even if
+  // it were valid, it wouldn't be the desired outcome as we would always load
+  // from addrspace(0), instead of the addrspace of the pointer.
+  if (stripAddrSpaces(v->getType()) == t) {
     return v;
   }
   assert(!isaStruct(t));
@@ -537,7 +524,8 @@ LLValue *DtoBitCast(LLValue *v, LLType *t, const llvm::Twine &name) {
 }
 
 LLConstant *DtoBitCast(LLConstant *v, LLType *t) {
-  if (v->getType() == t) {
+  // Refer to the explanation in the other DtoBitCast overloaded function.
+  if (stripAddrSpaces(v->getType()) == t) {
     return v;
   }
   return llvm::ConstantExpr::getBitCast(v, t);
