@@ -306,6 +306,9 @@ llvm::BasicBlock *CleanupScope::run(IRState &irs, llvm::BasicBlock *sourceBlock,
   if (!branchSelector) {
     // ... and have not created one yet, so do so now.
     branchSelector = new llvm::AllocaInst(llvm::Type::getInt32Ty(irs.context()),
+#if LDC_LLVM_VER >= 500
+                                          irs.module.getDataLayout().getAllocaAddrSpace(),
+#endif
                                           llvm::Twine("branchsel.") +
                                               beginBlock()->getName(),
                                           irs.topallocapoint());
@@ -617,21 +620,19 @@ TryCatchFinallyScopes::getLandingPadRef(CleanupCursor scope) {
 }
 
 namespace {
-llvm::LandingPadInst *createLandingPadInst(IRState &irs) {
-  LLType *retType =
+  llvm::LandingPadInst *createLandingPadInst(IRState &irs) {
+    LLType *retType =
       LLStructType::get(LLType::getInt8PtrTy(irs.context()),
-                        LLType::getInt32Ty(irs.context()), nullptr);
-#if LDC_LLVM_VER >= 307
+        LLType::getInt32Ty(irs.context())
+#if LDC_LLVM_VER < 500
+        , nullptr
+#endif
+      );
   if (!irs.func()->hasLLVMPersonalityFn()) {
     irs.func()->setLLVMPersonalityFn(
         getRuntimeFunction(Loc(), irs.module, "_d_eh_personality"));
   }
   return irs.ir->CreateLandingPad(retType, 0);
-#else
-  LLFunction *personalityFn =
-      getRuntimeFunction(Loc(), irs.module, "_d_eh_personality");
-  return irs.ir->CreateLandingPad(retType, personalityFn, 0);
-#endif
 }
 }
 
